"""数据库迁移管理。

提供类似 Django 的迁移管理接口，封装 Alembic 命令，并增强功能。
"""

from __future__ import annotations

import asyncio
from collections.abc import Callable
import importlib
import inspect
from pathlib import Path
import pkgutil
from typing import Any

from alembic import command
from alembic.autogenerate import compare_metadata
from alembic.config import Config
from alembic.runtime.migration import MigrationContext
from alembic.script import ScriptDirectory
from sqlalchemy import MetaData
from sqlalchemy.ext.asyncio import create_async_engine
from sqlalchemy.orm import DeclarativeBase

from aury.boot.common.logging import logger
from aury.boot.domain.models import Base


def load_all_models(model_modules: list[str]) -> None:
    """加载所有模型模块，确保 Alembic 可以检测到它们。
    
    这个函数会导入指定的模块及其所有子模块，确保所有继承自 Base 的模型类
    被正确注册到 SQLAlchemy 的元数据中。必须在 Alembic env.py 中调用此函数。
    
    支持的模式：
    - 精确模块名: "app.models"（既可以是文件 app/models.py，也可以是包 app/models/__init__.py）
    - 单层通配符: "app.*.models" 匹配 app.users.models, app.products.models
    - 递归通配符: "app.**" 递归匹配 app 下所有子模块
    - 混合模式: "app.**.models" 递归匹配所有以 models 结尾的模块
    
    Args:
        model_modules: 模型模块列表或模式，如 ["app.models", "app.**.models"]
        
    示例:
        # 在 alembic/env.py 中
        from aury.boot.application.migrations import load_all_models
        from aury.boot.domain.models import Base
        
        # 加载所有模型
        load_all_models(["app.models", "app.**.models"])
        
        # 确保使用 Base.metadata
        target_metadata = Base.metadata
    """
    if not model_modules:
        logger.warning("未配置 model_modules，Alembic 可能无法检测到模型变更")
        return
    
    loaded_count = 0
    
    for module_pattern in model_modules:
        # 检查是否有通配符
        if '*' in module_pattern:
            # 处理通配符模式
            modules = _expand_module_pattern(module_pattern)
        else:
            # 精确模块名
            modules = [module_pattern]
        
        for mod_name in modules:
            try:
                mod = importlib.import_module(mod_name)
                logger.debug(f"已导入模型模块: {mod_name}")
                loaded_count += 1
                
                # 如果是包，递归导入其所有子模块
                if hasattr(mod, '__path__'):
                    _load_package_submodules(mod)
                    
            except ImportError as e:
                logger.debug(f"无法导入模型模块 {mod_name}: {e}")
            except Exception as e:
                logger.debug(f"加载模型模块 {mod_name} 时出错: {e}")
    
    if loaded_count > 0:
        logger.info(f"✅ 已加载 {loaded_count} 个模型模块")
    else:
        logger.warning("⚠️  未加载任何模型模块，Alembic 可能无法检测到模型变更")


def _expand_module_pattern(pattern: str) -> list[str]:
    """根据通配符模式展开模块列表。
    
    支持的模式：
    - app.*.models: 匹配 app 下单层的 models 模块（app.users.models, app.products.models）
    - app.**.models: 递归匹配 app 下所有层的 models 模块
    - app.**: 递归匹配 app 下所有子模块
    
    Args:
        pattern: 包含通配符的模块模式
        
    Returns:
        展开后的模块名列表
    """
    modules = []
    
    if '**' in pattern:
        # 递归通配符：app.**.models 或 app.**
        # 提取基础包名（** 之前的部分）
        parts = pattern.split('**')
        base_pkg = parts[0].rstrip('.')
        suffix = parts[1].lstrip('.')  # models 或空字符串
        
        try:
            pkg = importlib.import_module(base_pkg)
            if hasattr(pkg, '__path__'):
                # 递归遍历所有子模块
                for _, modname, _ in pkgutil.walk_packages(
                    path=pkg.__path__,
                    prefix=f"{base_pkg}.",
                    onerror=lambda x: None,
                ):
                    # 如果有后缀，检查模块名是否以后缀结尾
                    if suffix:
                        if modname.endswith(suffix):
                            modules.append(modname)
                    else:
                        modules.append(modname)
        except ImportError:
            pass
            
    else:
        # 单层通配符：app.*.models
        # 提取基础包和后缀
        parts = pattern.split('*')
        base_pkg = parts[0].rstrip('.')
        suffix = parts[1].lstrip('.')
        
        try:
            pkg = importlib.import_module(base_pkg)
            if hasattr(pkg, '__path__'):
                # 只遍历一层子模块
                for _, modname, _ in pkgutil.iter_modules(pkg.__path__, prefix=f"{base_pkg}."):
                    # 检查是否有后缀部分，如果有则继续尝试导入
                    if suffix:
                        full_name = f"{modname}.{suffix}"
                        try:
                            importlib.import_module(full_name)
                            modules.append(full_name)
                        except ImportError:
                            pass
                    else:
                        modules.append(modname)
        except ImportError:
            pass
    
    return modules


def _load_package_submodules(package_module) -> None:
    """递归加载包内的所有模块。
    
    对于包（目录），这个函数会遍历其所有子模块并导入它们，
    确保其中定义的模型都被 SQLAlchemy 注册到 Base.metadata。
    
    Args:
        package_module: 已导入的包模块对象
    """
    if not hasattr(package_module, '__path__'):
        return
    
    package_name = package_module.__name__
    package_path = Path(package_module.__path__[0])
    
    # 遍历包目录下的所有 .py 文件（不包括 __pycache__）
    for py_file in package_path.glob('*.py'):
        # 跳过 __init__.py 和特殊文件
        if py_file.name.startswith('_'):
            continue
        
        module_name = py_file.stem
        full_module_name = f"{package_name}.{module_name}"
        
        try:
            importlib.import_module(full_module_name)
            logger.debug(f"已加载包中的模块: {full_module_name}")
        except ImportError as e:
            logger.debug(f"无法导入模块 {full_module_name}: {e}")
        except Exception as e:
            logger.debug(f"加载模块 {full_module_name} 时出错: {e}")


class MigrationManager:
    """迁移管理器。
    
    提供类似 Django 的迁移管理接口，并增强功能：
    - 自动检测模型变更
    - 数据迁移支持
    - 迁移前/后钩子
    - 干运行（dry-run）
    - 迁移检查
    - 更好的错误处理
    - 自动创建配置和目录
    
    使用示例:
        manager = MigrationManager(
            config_path="alembic.ini",
            script_location="migrations",
            database_url="sqlite+aiosqlite:///./app.db",
            model_modules=["app.models"],
            auto_create=True,
        )
        
        # 生成迁移（自动检测模型变更）
        await manager.make_migrations(message="add user table")
        
        # 执行迁移（带钩子）
        await manager.upgrade()
        
        # 查看状态
        status = await manager.status()
    """
    
    def __init__(
        self,
        database_url: str,
        config_path: str = "alembic.ini",
        script_location: str = "migrations",
        model_modules: list[str] | None = None,
        auto_create: bool = True,
    ) -> None:
        """初始化迁移管理器。
        
        Args:
            database_url: 数据库连接字符串
            config_path: Alembic 配置文件路径
            script_location: Alembic 迁移脚本目录
            model_modules: 模型模块列表（用于自动检测变更）
            auto_create: 是否自动创建配置和目录
        """
        self._database_url = database_url
        self._config_path = Path(config_path)
        self._script_location = script_location
        self._model_modules = model_modules or []
        
        # 自动创建配置和目录
        if auto_create:
            self._ensure_migration_setup()
        
        # 加载 Alembic 配置
        if not self._config_path.exists():
            raise FileNotFoundError(
                f"Alembic 配置文件不存在: {config_path}\n"
                f"请设置 auto_create=True 以自动创建配置"
            )
        
        self._alembic_cfg = Config(str(self._config_path))
        self._alembic_cfg.set_main_option("script_location", self._script_location)
        self._alembic_cfg.set_main_option("sqlalchemy.url", self._database_url)
        
        # 迁移钩子
        self._before_upgrade_hooks: list[Callable[[str], None]] = []
        self._after_upgrade_hooks: list[Callable[[str], None]] = []
        self._before_downgrade_hooks: list[Callable[[str], None]] = []
        self._after_downgrade_hooks: list[Callable[[str], None]] = []
        
        logger.debug(f"迁移管理器已初始化: {config_path}")
    
    def _ensure_migration_setup(self) -> None:
        """确保迁移配置和目录存在，不存在则自动创建。
        
        使用统一的 setup 模块，保证单一数据源。
        """
        from .setup import ensure_migration_setup
        
        # 获取当前工作目录作为 base_path
        base_path = Path.cwd()
        
        ensure_migration_setup(
            base_path=base_path,
            config_path=str(self._config_path),
            script_location=self._script_location,
            model_modules=self._model_modules,
        )
    
    def register_before_upgrade(self, hook: Callable[[str], None]) -> None:
        """注册升级前钩子。
        
        Args:
            hook: 钩子函数，接收目标版本作为参数
        """
        self._before_upgrade_hooks.append(hook)
    
    def register_after_upgrade(self, hook: Callable[[str], None]) -> None:
        """注册升级后钩子。
        
        Args:
            hook: 钩子函数，接收目标版本作为参数
        """
        self._after_upgrade_hooks.append(hook)
    
    def register_before_downgrade(self, hook: Callable[[str], None]) -> None:
        """注册回滚前钩子。
        
        Args:
            hook: 钩子函数，接收目标版本作为参数
        """
        self._before_downgrade_hooks.append(hook)
    
    def register_after_downgrade(self, hook: Callable[[str], None]) -> None:
        """注册回滚后钩子。
        
        Args:
            hook: 钩子函数，接收目标版本作为参数
        """
        self._after_downgrade_hooks.append(hook)
    
    def _load_models(self) -> set[type[DeclarativeBase]]:
        """加载所有模型（用于自动检测变更）。
        
        Returns:
            set[type[DeclarativeBase]]: 模型类集合
        """
        models: set[type[DeclarativeBase]] = set()
        
        for module_name in self._model_modules:
            try:
                module = importlib.import_module(module_name)
                for name, obj in inspect.getmembers(module):
                    if (
                        inspect.isclass(obj)
                        and issubclass(obj, Base)
                        and obj is not Base
                        and not getattr(obj, "__abstract__", False)
                    ):
                        models.add(obj)
                        logger.debug(f"加载模型: {module_name}.{name}")
            except ImportError as e:
                logger.warning(f"无法导入模型模块 {module_name}: {e}")
        
        return models
    
    async def _detect_changes(self) -> list[dict[str, Any]]:
        """检测模型变更（类似 Django 的 autodetect）。
        
        Returns:
            list[dict[str, Any]]: 变更列表
        """
        if not self._model_modules:
            return []
        
        try:
            models = self._load_models()
            if not models:
                return []
            
            # 使用异步引擎
            engine = create_async_engine(self._database_url)
            
            def _sync_detect(conn):
                context = MigrationContext.configure(conn)
                diff = compare_metadata(context, Base.metadata)
                changes = []
                for change in diff:
                    changes.append({
                        "type": type(change).__name__,
                        "description": str(change),
                    })
                return changes
            
            async with engine.connect() as conn:
                changes = await conn.run_sync(_sync_detect)
            
            await engine.dispose()
            return changes
        except Exception as e:
            logger.warning(f"检测模型变更失败: {e}")
            return []
    
    async def check(self) -> dict[str, Any]:
        """检查迁移（类似 Django 的 check）。
        
        检查迁移文件是否有问题，如：
        - 迁移依赖是否正确
        - 是否有冲突
        - 是否有缺失的迁移
        
        Returns:
            dict[str, Any]: 检查结果
        """
        def _check():
            script = ScriptDirectory.from_config(self._alembic_cfg)
            revisions = list(script.walk_revisions())
            
            issues = []
            warnings = []
            
            # 检查是否有孤立的迁移
            revision_map = {rev.revision: rev for rev in revisions}
            for rev in revisions:
                if rev.down_revision and rev.down_revision not in revision_map:
                    issues.append(f"迁移 {rev.revision} 的父版本 {rev.down_revision} 不存在")
            
            # 检查是否有多个 head（冲突）
            heads = script.get_revisions("heads")
            if len(heads) > 1:
                warnings.append(f"发现 {len(heads)} 个 head，可能存在分支，需要合并")
            
            return {
                "valid": len(issues) == 0,
                "issues": issues,
                "warnings": warnings,
                "revision_count": len(revisions),
                "head_count": len(heads),
            }
        
        return await asyncio.to_thread(_check)
    
    async def make_migrations(
        self,
        message: str | None = None,
        autogenerate: bool = True,
        dry_run: bool = False,
    ) -> dict[str, Any]:
        """生成迁移文件（类似 Django 的 makemigrations）。
        
        Args:
            message: 迁移消息
            autogenerate: 是否自动生成（基于模型变更）
            dry_run: 是否干运行（只检测变更，不生成文件）
            
        Returns:
            dict[str, Any]: 生成结果，包含变更信息
        """
        if not message:
            message = "auto migration"
        
        # 检测变更
        changes = []
        if autogenerate and self._model_modules:
            changes = await self._detect_changes()
            if changes:
                logger.info(f"检测到 {len(changes)} 个模型变更")
                for change in changes:
                    logger.debug(f"  - {change['type']}: {change['description']}")
        
        if dry_run:
            return {
                "dry_run": True,
                "changes": changes,
                "message": message,
            }
        
        def _make():
            command.revision(
                self._alembic_cfg,
                message=message,
                autogenerate=autogenerate,
            )
        
        await asyncio.to_thread(_make)
        logger.info(f"迁移文件已生成: {message}")
        
        return {
            "dry_run": False,
            "changes": changes,
            "message": message,
            "path": f"{self._script_location}/versions/{message.replace(' ', '_')}.py",
        }
    
    async def upgrade(
        self,
        revision: str = "head",
        dry_run: bool = False,
    ) -> None:
        """执行迁移（类似 Django 的 migrate）。
        
        Args:
            revision: 目标版本（默认 "head" 表示最新版本）
            dry_run: 是否干运行（只显示会执行的迁移，不实际执行）
        """
        if dry_run:
            # 干运行：只显示会执行的迁移
            status_info = await self.status()
            pending = status_info.get("pending", [])
            if pending:
                logger.info(f"干运行：将执行 {len(pending)} 个迁移")
                for rev in pending:
                    logger.info(f"  - {rev}")
            else:
                logger.info("干运行：没有待执行的迁移")
            return
        
        # 执行钩子
        for hook in self._before_upgrade_hooks:
            try:
                hook(revision)
            except Exception as e:
                logger.error(f"升级前钩子执行失败: {e}")
        
        def _upgrade():
            command.upgrade(self._alembic_cfg, revision)
        
        await asyncio.to_thread(_upgrade)
        logger.info(f"迁移已执行到版本: {revision}")
        
        # 执行钩子
        for hook in self._after_upgrade_hooks:
            try:
                hook(revision)
            except Exception as e:
                logger.error(f"升级后钩子执行失败: {e}")
    
    async def downgrade(
        self,
        revision: str,
        dry_run: bool = False,
    ) -> None:
        """回滚迁移。
        
        Args:
            revision: 目标版本（如 "previous", "-1", 或具体版本号）
            dry_run: 是否干运行（只显示会回滚的迁移，不实际执行）
        """
        if dry_run:
            # 干运行：显示会回滚的迁移
            status_info = await self.status()
            current = status_info.get("current")
            if current:
                logger.info(f"干运行：将从 {current} 回滚到 {revision}")
            return
        
        # 执行钩子
        for hook in self._before_downgrade_hooks:
            try:
                hook(revision)
            except Exception as e:
                logger.error(f"回滚前钩子执行失败: {e}")
        
        def _downgrade():
            command.downgrade(self._alembic_cfg, revision)
        
        await asyncio.to_thread(_downgrade)
        logger.info(f"迁移已回滚到版本: {revision}")
        
        # 执行钩子
        for hook in self._after_downgrade_hooks:
            try:
                hook(revision)
            except Exception as e:
                logger.error(f"回滚后钩子执行失败: {e}")
    
    async def status(self) -> dict[str, Any]:
        """查看迁移状态（类似 Django 的 showmigrations）。
        
        Returns:
            dict[str, Any]: 迁移状态信息
        """
        script = ScriptDirectory.from_config(self._alembic_cfg)
        
        # 使用异步引擎，通过 run_sync 执行同步操作
        engine = create_async_engine(self._database_url)
        current_rev = None
        
        try:
            async with engine.connect() as conn:
                def _get_current_rev(connection):
                    context = MigrationContext.configure(connection)
                    return context.get_current_revision()
                
                current_rev = await conn.run_sync(_get_current_rev)
        finally:
            await engine.dispose()
        
        head_rev = script.get_current_head()
        revisions = list(script.walk_revisions())
        
        applied = []
        pending = []
        
        # 简化逻辑：
        # - 如果 current_rev 是 None，所有迁移都是 pending
        # - 如果 current_rev == head_rev，没有 pending
        # - 否则，从 head 到 current 之间的是 pending
        
        if current_rev is None:
            # 数据库是新的，所有迁移都需要执行
            pending = [rev.revision for rev in revisions]
        elif current_rev == head_rev:
            # 已是最新版本
            applied = [rev.revision for rev in revisions]
        else:
            # 部分已执行，部分待执行
            # 从 head 向下遍历，直到 current_rev
            found_current = False
            for rev in revisions:  # walk_revisions 从 head 开始
                if rev.revision == current_rev:
                    found_current = True
                    applied.append(rev.revision)
                elif found_current:
                    applied.append(rev.revision)
                else:
                    pending.append(rev.revision)
        
        return {
            "current": current_rev,
            "head": head_rev,
            "pending": pending,
            "applied": applied,
        }
    
    async def show(self) -> list[dict[str, str]]:
        """显示所有迁移（类似 Django 的 showmigrations）。
        
        Returns:
            list[dict[str, str]]: 迁移列表
        """
        def _show():
            script = ScriptDirectory.from_config(self._alembic_cfg)
            revisions = list(script.walk_revisions())
            
            result = []
            for rev in revisions:
                result.append({
                    "revision": rev.revision,
                    "down_revision": rev.down_revision,
                    "message": rev.doc or "",
                    "path": str(rev.path) if hasattr(rev, "path") else "",
                })
            
            return result
        
        return await asyncio.to_thread(_show)
    
    async def history(self, verbose: bool = False) -> list[str]:
        """显示迁移历史。
        
        Args:
            verbose: 是否显示详细信息
            
        Returns:
            list[str]: 迁移历史列表
        """
        def _history():
            command.history(self._alembic_cfg, verbose=verbose)
        
        await asyncio.to_thread(_history)
        return []
    
    async def merge(
        self,
        revisions: list[str],
        message: str | None = None,
    ) -> str:
        """合并迁移（类似 Django 的迁移合并）。
        
        当有多个分支时，创建合并迁移。
        
        Args:
            revisions: 要合并的版本列表
            message: 合并消息
            
        Returns:
            str: 合并后的迁移文件路径
        """
        if not message:
            message = f"merge {', '.join(revisions)}"
        
        def _merge():
            command.merge(
                self._alembic_cfg,
                revisions=revisions,
                message=message,
            )
        
        await asyncio.to_thread(_merge)
        logger.info(f"迁移已合并: {message}")
        return f"{self._script_location}/versions/{message.replace(' ', '_')}.py"
    
    async def stamp(
        self,
        revision: str,
        purge: bool = False,
    ) -> None:
        """标记数据库版本（不执行迁移）。
        
        用于修复数据库版本不匹配的问题，例如当迁移文件被删除但数据库
        已记录该版本时。这个命令只更新 alembic_version 表，不执行
        任何实际的迁移操作。
        
        Args:
            revision: 目标版本（如 "head", "base", 或具体版本号）
            purge: 是否清除 alembic_version 表中的所有记录后再标记
        """
        def _stamp():
            command.stamp(self._alembic_cfg, revision, purge=purge)
        
        await asyncio.to_thread(_stamp)
        logger.info(f"数据库版本已标记为: {revision}")


__all__ = [
    "MigrationManager",
    "load_all_models",
]




"""RPC基类和异常定义。"""

from __future__ import annotations

from typing import Any, Optional

from pydantic import BaseModel


class RPCError(Exception):
    """RPC调用异常。"""

    def __init__(
        self,
        message: str,
        code: str = "RPC_ERROR",
        status_code: int = 500,
        details: dict[str, Any] | None = None,
    ) -> None:
        """初始化RPC异常。

        Args:
            message: 错误消息
            code: 错误代码
            status_code: HTTP状态码
            details: 错误详情
        """
        super().__init__(message)
        self.message = message
        self.code = code
        self.status_code = status_code
        self.details = details or {}

    def __str__(self) -> str:
        return f"[{self.code}] {self.message} (status: {self.status_code})"


class RPCResponse(BaseModel):
    """RPC响应模型。"""

    success: bool = True
    data: Any | None = None
    message: str = ""
    code: str = "0000"
    status_code: int = 200

    def raise_for_status(self) -> None:
        """如果响应失败，抛出异常。"""
        if not self.success:
            raise RPCError(
                message=self.message,
                code=self.code,
                status_code=self.status_code,
            )


class BaseRPCClient:
    """RPC客户端基类。

    提供统一的RPC调用接口和错误处理。
    """

    def __init__(
        self,
        base_url: str,
        timeout: int = 30,
        retry_times: int = 3,
        headers: dict[str, str] | None = None,
    ) -> None:
        """初始化RPC客户端。

        Args:
            base_url: 服务基础URL
            timeout: 超时时间（秒）
            retry_times: 重试次数
            headers: 默认请求头
        """
        self.base_url = base_url.rstrip("/")
        self.timeout = timeout
        self.retry_times = retry_times
        self.headers = headers or {}

    def _build_url(self, path: str) -> str:
        """构建完整URL。

        Args:
            path: API路径

        Returns:
            str: 完整URL
        """
        path = path.lstrip("/")
        return f"{self.base_url}/{path}"

    def _prepare_headers(self, extra_headers: dict[str, str] | None = None) -> dict[str, str]:
        """准备请求头。

        Args:
            extra_headers: 额外的请求头

        Returns:
            dict[str, str]: 合并后的请求头
        """
        headers = self.headers.copy()
        if extra_headers:
            headers.update(extra_headers)
        return headers




"""Aury 命名空间包（pkgutil namespace）。

`aury` 顶层命名空间可能同时由多个分发包提供（例如 `aury-sdk-storage`、`aury-boot`）。
为了避免其中任意一个分发包提供 `aury/__init__.py` 后把 `aury` 变成"普通包"
（从而阻止其它 sys.path 上的 `aury.*` 子包被发现），这里显式使用
`pkgutil.extend_path` 将其转换为"pkgutil 命名空间包"，允许多分发共存。
"""

from pkgutil import extend_path

__path__ = extend_path(__path__, __name__)



"""服务器管理命令。

提供 CLI 接口来管理和运行应用服务器。

使用示例:
    python -m aury.boot.commands.server run \\
        --host 0.0.0.0 \\
        --port 8000 \\
        --workers 4 \\
        --reload
"""

from .app import app, server_cli

__all__ = [
    "app",
    "server_cli",
]






"""迁移命令行工具。

提供数据库迁移管理的命令行接口。
"""

from .app import app, get_manager
from .commands import (
    check,
    down,
    history,
    make,
    merge,
    show,
    stamp,
    status,
    up,
)


def main() -> None:
    """命令行入口函数。"""
    app()


__all__ = [
    "app",
    "check",
    "down",
    "get_manager",
    "history",
    "main",
    "make",
    "merge",
    "show",
    "stamp",
    "status",
    "up",
]




"""第三方接口适配器异常定义。

用于第三方接口调用过程中的各类异常场景，所有异常都继承自 FoundationError。

异常类型：
- AdapterError: 通用错误（调用失败、第三方返回错误码等）
- AdapterDisabledError: 接口被禁用（mode=disabled 或 enabled=False）
- AdapterTimeoutError: 调用超时
- AdapterValidationError: 响应数据格式验证失败
"""

from __future__ import annotations

from typing import Any

from aury.boot.common.exceptions import FoundationError


class AdapterError(FoundationError):
    """第三方适配器通用错误。
    
    当第三方调用失败、返回错误码、数据解析异常等情况时抛出。
    
    Attributes:
        adapter_name: 适配器名称
        operation: 操作名称
        third_party_code: 第三方返回的错误码（如有）
        third_party_message: 第三方返回的错误消息（如有）
    
    使用示例:
        raise AdapterError(
            "支付失败",
            adapter_name="payment",
            operation="charge",
            third_party_code="INSUFFICIENT_FUNDS",
            third_party_message="余额不足",
        )
    """
    
    def __init__(
        self,
        message: str,
        *args: object,
        adapter_name: str | None = None,
        operation: str | None = None,
        third_party_code: str | None = None,
        third_party_message: str | None = None,
        metadata: dict[str, Any] | None = None,
        cause: Exception | None = None,
    ) -> None:
        """初始化集成异常。
        
        Args:
            message: 错误消息
            adapter_name: 适配器名称
            operation: 操作名称
            third_party_code: 第三方错误码
            third_party_message: 第三方错误消息
            metadata: 额外元数据
            cause: 原始异常
        """
        self.adapter_name = adapter_name
        self.operation = operation
        self.third_party_code = third_party_code
        self.third_party_message = third_party_message
        
        # 构建完整的元数据
        full_metadata = metadata or {}
        if adapter_name:
            full_metadata["adapter_name"] = adapter_name
        if operation:
            full_metadata["operation"] = operation
        if third_party_code:
            full_metadata["third_party_code"] = third_party_code
        if third_party_message:
            full_metadata["third_party_message"] = third_party_message
        
        super().__init__(message, *args, metadata=full_metadata, cause=cause)
    
    def __str__(self) -> str:
        """返回异常字符串表示。"""
        parts = [self.message]
        if self.adapter_name:
            parts.append(f"adapter={self.adapter_name}")
        if self.operation:
            parts.append(f"operation={self.operation}")
        if self.third_party_code:
            parts.append(f"code={self.third_party_code}")
        return " | ".join(parts)


class AdapterDisabledError(AdapterError):
    """当前环境禁用了某个第三方功能。
    
    当 mode=disabled 或 enabled=False 时，调用任何操作都会抛出此异常。
    
    使用示例:
        # 在测试环境禁用支付功能
        settings = AdapterSettings(mode="disabled")
        adapter = PaymentAdapter("payment", settings)
        
        await adapter.charge(100)  # 抛出 AdapterDisabledError
    """
    
    def __init__(
        self,
        message: str,
        *args: object,
        adapter_name: str | None = None,
        operation: str | None = None,
        metadata: dict[str, Any] | None = None,
    ) -> None:
        """初始化禁用异常。"""
        super().__init__(
            message,
            *args,
            adapter_name=adapter_name,
            operation=operation,
            metadata=metadata,
        )


class AdapterTimeoutError(AdapterError):
    """第三方调用超时。
    
    当调用第三方接口超过配置的超时时间时抛出。
    
    Attributes:
        timeout_seconds: 超时时间（秒）
    """
    
    def __init__(
        self,
        message: str,
        *args: object,
        adapter_name: str | None = None,
        operation: str | None = None,
        timeout_seconds: float | None = None,
        metadata: dict[str, Any] | None = None,
        cause: Exception | None = None,
    ) -> None:
        """初始化超时异常。"""
        self.timeout_seconds = timeout_seconds
        
        full_metadata = metadata or {}
        if timeout_seconds is not None:
            full_metadata["timeout_seconds"] = timeout_seconds
        
        super().__init__(
            message,
            *args,
            adapter_name=adapter_name,
            operation=operation,
            metadata=full_metadata,
            cause=cause,
        )


class AdapterValidationError(AdapterError):
    """第三方响应数据验证失败。
    
    当第三方返回的数据格式不符合预期时抛出。
    """
    
    def __init__(
        self,
        message: str,
        *args: object,
        adapter_name: str | None = None,
        operation: str | None = None,
        expected_format: str | None = None,
        actual_data: Any | None = None,
        metadata: dict[str, Any] | None = None,
        cause: Exception | None = None,
    ) -> None:
        """初始化验证异常。"""
        full_metadata = metadata or {}
        if expected_format:
            full_metadata["expected_format"] = expected_format
        if actual_data is not None:
            # 截断过长的数据
            actual_str = str(actual_data)
            if len(actual_str) > 200:
                actual_str = actual_str[:200] + "..."
            full_metadata["actual_data"] = actual_str
        
        super().__init__(
            message,
            *args,
            adapter_name=adapter_name,
            operation=operation,
            metadata=full_metadata,
            cause=cause,
        )


__all__ = [
    "AdapterError",
    "AdapterDisabledError",
    "AdapterTimeoutError",
    "AdapterValidationError",
]



"""Worker 命令 - 运行任务队列 Worker。

使用示例：
    aury worker                  # 运行 worker
    aury worker --app main:app   # 指定应用模块
    aury worker -c 4             # 指定并发数
"""

from __future__ import annotations

from pathlib import Path
import sys

from rich.console import Console
import typer

console = Console()

app = typer.Typer(
    name="worker",
    help="⚙️  运行任务队列 Worker",
    no_args_is_help=False,
)


def _detect_app_module() -> str:
    """自动检测应用模块路径。"""
    import os

    # 1. 环境变量
    if app_module := os.environ.get("APP_MODULE"):
        return app_module

    # 2. pyproject.toml 中的 [tool.aury] 配置
    pyproject_path = Path.cwd() / "pyproject.toml"
    if pyproject_path.exists():
        try:
            import tomllib
            with open(pyproject_path, "rb") as f:
                data = tomllib.load(f)
            if package := data.get("tool", {}).get("aury", {}).get("package"):
                return f"{package}.main:app"
        except Exception:
            pass

    # 3. 默认
    return "main:app"


@app.callback(invoke_without_command=True)
def run_worker(
    ctx: typer.Context,
    app_path: str | None = typer.Option(
        None,
        "--app",
        "-a",
        help="应用模块路径（默认自动检测，如 main:app）",
    ),
    concurrency: int = typer.Option(
        4,
        "--concurrency",
        "-c",
        help="并发 worker 数量",
    ),
    queues: str | None = typer.Option(
        None,
        "--queues",
        "-q",
        help="要处理的队列名称（逗号分隔）",
    ),
) -> None:
    """运行任务队列 Worker 进程。

    Worker 会消费任务队列中的异步任务并执行。

    示例：
        aury worker                       # 默认配置
        aury worker --app main:app        # 指定应用模块
        aury worker -c 8                  # 8 个并发
        aury worker -q high,default       # 只处理指定队列
    """
    if ctx.invoked_subcommand is not None:
        return

    # 确保当前目录在 Python 路径中
    cwd = str(Path.cwd())
    if cwd not in sys.path:
        sys.path.insert(0, cwd)

    app_module = app_path or _detect_app_module()
    console.print("[bold cyan]⚙️  启动 Worker[/bold cyan]")
    console.print(f"   应用: [green]{app_module}[/green]")
    console.print(f"   并发: [green]{concurrency}[/green]")
    if queues:
        console.print(f"   队列: [green]{queues}[/green]")

    try:
        # 导入应用（确保任务被注册）
        module_path, app_name = app_module.rsplit(":", 1)
        module = __import__(module_path, fromlist=[app_name])
        application = getattr(module, app_name)

        # 设置日志（必须在其他操作之前）
        from aury.boot.common.logging import setup_logging
        setup_logging(
            log_level=getattr(application, "_config", None) and application._config.log.level or "INFO",
            service_type="worker",
        )

        # 尝试导入 dramatiq
        try:
            import dramatiq
            from dramatiq.cli import main as dramatiq_main
        except ImportError:
            console.print("[red]❌ dramatiq 未安装[/red]")
            console.print("[dim]请安装: uv add 'aury-boot[tasks]'[/dim]")
            raise typer.Exit(1)

        console.print("[bold green]✅ Worker 启动中...[/bold green]")
        console.print("[dim]按 Ctrl+C 停止[/dim]")

        # 构建 dramatiq 参数
        args = [
            module_path,  # 模块路径
            "--processes", "1",
            "--threads", str(concurrency),
        ]
        if queues:
            args.extend(["--queues", queues])

        # 运行 dramatiq worker
        sys.argv = ["dramatiq", *args]
        dramatiq_main()

    except KeyboardInterrupt:
        console.print("\n[yellow]Worker 已停止[/yellow]")
    except ImportError as e:
        console.print(f"[red]❌ 无法导入应用: {e}[/red]")
        console.print("[dim]请确保应用模块路径正确，如 main:app[/dim]")
        raise typer.Exit(1)
    except Exception as e:
        console.print(f"[red]❌ 启动失败: {e}[/red]")
        raise typer.Exit(1)


__all__ = ["app"]



"""Broadcaster 通道后端。

基于 Broadcaster 库实现，支持多种后端：
- memory:// - 内存（单进程，开发/测试用）
- redis:// - Redis Pub/Sub（多进程/分布式）
- kafka:// - Apache Kafka
- postgres:// - PostgreSQL LISTEN/NOTIFY

优势：
- 共享连接池，支持成千上万并发订阅
- 自动重连机制
- 统一 API，通过 URL scheme 切换后端
"""

from __future__ import annotations

import json
from collections.abc import AsyncIterator
from datetime import datetime

from broadcaster import Broadcast

from aury.boot.common.logging import logger

from ..base import ChannelMessage, IChannel


class BroadcasterChannel(IChannel):
    """Broadcaster 通道实现。

    使用 Broadcaster 库统一处理多种后端，解决原生实现的连接池问题：
    - 共享单个连接处理所有订阅
    - 内部通过 asyncio.Queue 分发消息
    - 支持成千上万并发订阅者
    """

    def __init__(self, url: str) -> None:
        """初始化 Broadcaster 通道。

        Args:
            url: 连接 URL，支持的 scheme：
                - memory:// - 内存后端
                - redis://host:port/db - Redis Pub/Sub
                - kafka://host:port - Apache Kafka
                - postgres://user:pass@host:port/db - PostgreSQL
        """
        self._url = url
        self._broadcast = Broadcast(url)
        self._connected = False

    async def _ensure_connected(self) -> None:
        """确保已连接。"""
        if not self._connected:
            await self._broadcast.connect()
            self._connected = True
            logger.debug(f"Broadcaster 通道已连接: {self._mask_url(self._url)}")

    def _mask_url(self, url: str) -> str:
        """URL 脱敏（隐藏密码）。"""
        if "@" in url:
            parts = url.split("@")
            prefix = parts[0]
            suffix = parts[1]
            if ":" in prefix:
                scheme_and_user = prefix.rsplit(":", 1)[0]
                return f"{scheme_and_user}:***@{suffix}"
        return url

    async def publish(self, channel: str, message: ChannelMessage) -> None:
        """发布消息到通道。"""
        await self._ensure_connected()

        message.channel = channel
        # 序列化消息
        data = {
            "data": message.data,
            "event": message.event,
            "id": message.id,
            "channel": message.channel,
            "timestamp": message.timestamp.isoformat(),
        }
        await self._broadcast.publish(channel=channel, message=json.dumps(data))

    async def subscribe(self, channel: str) -> AsyncIterator[ChannelMessage]:
        """订阅通道。

        Broadcaster 内部共享连接，每个订阅者不会创建新的连接。
        """
        await self._ensure_connected()

        async with self._broadcast.subscribe(channel=channel) as subscriber:
            async for event in subscriber:
                try:
                    data = json.loads(event.message)
                    message = ChannelMessage(
                        data=data.get("data"),
                        event=data.get("event"),
                        id=data.get("id"),
                        channel=data.get("channel") or channel,
                        timestamp=datetime.fromisoformat(data["timestamp"])
                        if data.get("timestamp")
                        else datetime.now(),
                    )
                    yield message
                except (json.JSONDecodeError, KeyError, TypeError) as e:
                    logger.warning(f"解析通道消息失败: {e}")

    async def psubscribe(self, pattern: str) -> AsyncIterator[ChannelMessage]:
        """模式订阅（通配符）。

        注意：Broadcaster 目前不支持模式订阅，此方法会抛出 NotImplementedError。
        如需模式订阅，请使用具体的 channel 名称。

        Args:
            pattern: 通道模式

        Raises:
            NotImplementedError: Broadcaster 不支持模式订阅
        """
        raise NotImplementedError(
            "Broadcaster 后端不支持模式订阅 (psubscribe)。"
            "请使用具体的 channel 名称。"
        )

    async def unsubscribe(self, channel: str) -> None:
        """取消订阅通道。

        注意：Broadcaster 的订阅通过上下文管理器自动处理，
        退出 subscribe() 的 async for 循环即可取消订阅。
        """
        pass

    async def close(self) -> None:
        """关闭通道。"""
        if self._connected:
            await self._broadcast.disconnect()
            self._connected = False
            logger.debug("Broadcaster 通道已关闭")


__all__ = ["BroadcasterChannel"]



"""Aury Boot 统一命令行入口。

提供统一的 CLI 入口，整合所有子命令：
- aury init              项目脚手架初始化
- aury generate          代码生成器
- aury server            服务器管理
- aury scheduler         独立运行调度器
- aury worker            运行任务队列 Worker
- aury migrate           数据库迁移
- aury docker            Docker 配置生成
- aury docs              生成/更新项目文档

使用示例：
    aury init                      # 初始化项目
    aury generate crud user        # 生成 CRUD 代码
    aury server dev                # 启动开发服务器
    aury scheduler                 # 独立运行调度器
    aury worker                    # 运行 Worker
    aury migrate up                # 执行数据库迁移
    aury docs all --force          # 更新所有文档

CLI 继承：
    子框架（如 aury-django、aury-cloud）可以通过 `register_commands` 继承所有基础命令：
    
    ```python
    from typer import Typer
    from aury.boot.commands import register_commands
    
    app = Typer(name="aury-django")
    register_commands(app)  # 继承所有 aury-boot 命令
    
    # 添加 django 特有命令
    @app.command()
    def startapp(name: str):
        ...
    ```
"""

from __future__ import annotations

import typer

app: typer.Typer | None = None
_registered = False


def _get_app() -> typer.Typer:
    """获取并初始化 Typer 应用（延迟加载）。"""
    global app, _registered
    
    if app is None:
        app = typer.Typer(
            name="aury",
            help="🚀 Aury Boot CLI - 现代化微服务开发工具",
            add_completion=True,
            no_args_is_help=True,
            rich_markup_mode="rich",
        )
        
        @app.callback(invoke_without_command=True)
        def callback(
            ctx: typer.Context,
            version: bool = typer.Option(
                False,
                "--version",
                "-v",
                help="显示版本信息",
                is_eager=True,
            ),
        ) -> None:
            """Aury Boot - 现代化微服务基础架构框架。"""
            if version:
                from rich.console import Console

                from aury.boot import __version__
                console = Console()
                console.print(f"[bold cyan]Aury Boot[/bold cyan] v{__version__}")
                raise typer.Exit()
    
    if not _registered:
        _registered = True
        # 延迟导入子命令
        from .add import app as add_app
        from .docker import app as docker_app
        from .docs import app as docs_app
        from .generate import app as generate_app
        from .init import init
        from .migrate import app as migrate_app
        from .pkg import app as pkg_app
        from .scheduler import app as scheduler_app
        from .server import app as server_app
        from .worker import app as worker_app

        app.command(name="init", help="🎯 初始化项目脚手架")(init)
        app.add_typer(add_app, name="add", help="➕ 添加可选模块")
        app.add_typer(pkg_app, name="pkg", help="📦 包管理")
        app.add_typer(generate_app, name="generate", help="⚡ 代码生成器")
        app.add_typer(server_app, name="server", help="🖥️  服务器管理")
        app.add_typer(scheduler_app, name="scheduler", help="🕐 独立运行调度器")
        app.add_typer(worker_app, name="worker", help="⚙️  运行任务队列 Worker")
        app.add_typer(migrate_app, name="migrate", help="🗃️  数据库迁移")
        app.add_typer(docker_app, name="docker", help="🐳 Docker 配置")
        app.add_typer(docs_app, name="docs", help="📚 生成/更新项目文档")
    
    return app


def main() -> None:
    """CLI 入口点。"""
    _get_app()()


def register_commands(
    target_app: typer.Typer,
    *,
    include_init: bool = True,
    include_add: bool = True,
    include_generate: bool = True,
    include_server: bool = True,
    include_scheduler: bool = True,
    include_worker: bool = True,
    include_migrate: bool = True,
    include_docker: bool = True,
    include_docs: bool = True,
) -> None:
    """将 aury-boot 的所有命令注册到目标 Typer app。
    
    用于子框架（如 aury-django、aury-cloud）继承基础命令。
    
    Args:
        target_app: 目标 Typer 应用
        include_*: 是否包含对应的命令组
    
    使用示例:
        ```python
        from typer import Typer
        from aury.boot.commands import register_commands
        
        app = Typer(name="aury-django")
        
        # 继承所有 aury-boot 命令
        register_commands(app)
        
        # 或选择性继承
        register_commands(app, include_docker=False)
        
        # 添加 django 特有命令
        django_app = Typer(name="django")
        
        @django_app.command()
        def startapp(name: str):
            '''Django startapp'''
            ...
        
        app.add_typer(django_app, name="django")
        ```
    """
    # 延迟导入子命令
    if include_init:
        from .init import init
        target_app.command(name="init", help="🎯 初始化项目脚手架")(init)
    
    if include_add:
        from .add import app as add_app
        target_app.add_typer(add_app, name="add", help="➕ 添加可选模块")
    
    # pkg 命令始终注册（包管理是通用功能）
    from .pkg import app as pkg_app
    target_app.add_typer(pkg_app, name="pkg", help="📦 包管理")
    
    if include_generate:
        from .generate import app as generate_app
        target_app.add_typer(generate_app, name="generate", help="⚡ 代码生成器")
    
    if include_server:
        from .server import app as server_app
        target_app.add_typer(server_app, name="server", help="🖥️  服务器管理")
    
    if include_scheduler:
        from .scheduler import app as scheduler_app
        target_app.add_typer(scheduler_app, name="scheduler", help="🕐 独立运行调度器")
    
    if include_worker:
        from .worker import app as worker_app
        target_app.add_typer(worker_app, name="worker", help="⚙️  运行任务队列 Worker")
    
    if include_migrate:
        from .migrate import app as migrate_app
        target_app.add_typer(migrate_app, name="migrate", help="🗃️  数据库迁移")
    
    if include_docker:
        from .docker import app as docker_app
        target_app.add_typer(docker_app, name="docker", help="🐳 Docker 配置")
    
    if include_docs:
        from .docs import app as docs_app
        target_app.add_typer(docs_app, name="docs", help="📚 生成/更新项目文档")


def get_command_modules() -> dict[str, type]:
    """获取所有命令模块，供子框架进一步定制。
    
    Returns:
        dict: 命令名 -> 模块对象
    
    使用示例:
        ```python
        from aury.boot.commands import get_command_modules
        
        modules = get_command_modules()
        # {'init': <module>, 'add': <module>, 'server': <module>, ...}
        ```
    """
    from . import add, docker, docs, generate, init, migrate, pkg, scheduler, server, worker
    
    return {
        "init": init,
        "add": add,
        "pkg": pkg,
        "generate": generate,
        "server": server,
        "scheduler": scheduler,
        "worker": worker,
        "migrate": migrate,
        "docker": docker,
        "docs": docs,
    }


# 允许 `from .app import app`
def __getattr__(name: str):
    if name == "app":
        return _get_app()
    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")


__all__ = [
    "app",
    "get_command_modules",
    "main",
    "register_commands",
]



"""客户端模块 - 外部服务连接管理。

提供统一的外部服务客户端，支持多实例。

包含:
- redis: Redis 客户端
- rabbitmq: RabbitMQ 客户端
"""

from __future__ import annotations

from .rabbitmq import RabbitMQClient, RabbitMQConfig
from .redis import RedisClient, RedisConfig

__all__ = [
    # RabbitMQ
    "RabbitMQClient",
    "RabbitMQConfig",
    # Redis
    "RedisClient",
    "RedisConfig",
]



"""告警聚合器。

实现累计触发和抑制逻辑：
- 滑动窗口计数：在窗口时间内达到阈值才触发
- 抑制机制：相同告警在抑制时间内不重复发送
"""

from __future__ import annotations

from collections import deque
from dataclasses import dataclass, field
from datetime import datetime, timedelta
import threading
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .events import AlertEvent


@dataclass
class AggregationState:
    """单个指纹的聚合状态。"""
    
    fingerprint: str
    
    # 滑动窗口内的事件时间戳
    event_timestamps: deque[datetime] = field(default_factory=deque)
    
    # 最后一次发送告警的时间
    last_alert_time: datetime | None = None
    
    # 聚合的 trace_id 列表（最多保留 5 个）
    trace_ids: deque[str] = field(default_factory=lambda: deque(maxlen=5))
    
    # 窗口内事件总数（用于通知）
    window_count: int = 0


class AlertAggregator:
    """告警聚合器。
    
    实现两层控制：
    1. 累计触发：窗口时间内达到阈值才触发告警
    2. 抑制机制：触发后的抑制时间内不重复发送
    
    示例：
        aggregator = AlertAggregator(
            window_seconds=60,      # 1 分钟窗口
            threshold=5,            # 5 次触发
            suppress_seconds=300,   # 5 分钟抑制
        )
        
        if aggregator.should_alert(event):
            # 发送告警
            ...
    """
    
    def __init__(
        self,
        window_seconds: int = 60,
        threshold: int = 1,
        suppress_seconds: int = 300,
    ) -> None:
        """初始化聚合器。
        
        Args:
            window_seconds: 滑动窗口大小（秒）
            threshold: 窗口内触发阈值
            suppress_seconds: 告警抑制时间（秒）
        """
        self.window_seconds = window_seconds
        self.threshold = threshold
        self.suppress_seconds = suppress_seconds
        
        self._states: dict[str, AggregationState] = {}
        self._lock = threading.Lock()
    
    def should_alert(self, event: "AlertEvent") -> bool:
        """判断是否应该触发告警。
        
        Args:
            event: 告警事件
        
        Returns:
            bool: 是否应该发送告警
        """
        fingerprint = event.fingerprint
        now = event.timestamp
        
        with self._lock:
            # 获取或创建状态
            if fingerprint not in self._states:
                self._states[fingerprint] = AggregationState(fingerprint=fingerprint)
            
            state = self._states[fingerprint]
            
            # 清理过期的时间戳
            window_start = now - timedelta(seconds=self.window_seconds)
            while state.event_timestamps and state.event_timestamps[0] < window_start:
                state.event_timestamps.popleft()
            
            # 添加当前事件
            state.event_timestamps.append(now)
            state.trace_ids.append(event.trace_id)
            state.window_count = len(state.event_timestamps)
            
            # 检查是否达到阈值
            if state.window_count < self.threshold:
                return False
            
            # 检查是否在抑制期内
            if state.last_alert_time:
                suppress_until = state.last_alert_time + timedelta(seconds=self.suppress_seconds)
                if now < suppress_until:
                    return False
            
            # 触发告警，更新最后告警时间
            state.last_alert_time = now
            return True
    
    def get_state(self, fingerprint: str) -> AggregationState | None:
        """获取指纹的聚合状态。
        
        Args:
            fingerprint: 事件指纹
        
        Returns:
            聚合状态，如果不存在返回 None
        """
        with self._lock:
            return self._states.get(fingerprint)
    
    def get_aggregation_info(self, event: "AlertEvent") -> dict:
        """获取聚合信息（用于告警通知）。
        
        Args:
            event: 告警事件
        
        Returns:
            包含聚合信息的字典
        """
        state = self.get_state(event.fingerprint)
        if not state:
            return {
                "count": 1,
                "trace_ids": [event.trace_id],
            }
        
        return {
            "count": state.window_count,
            "trace_ids": list(state.trace_ids),
        }
    
    def reset(self, fingerprint: str | None = None) -> None:
        """重置聚合状态。
        
        Args:
            fingerprint: 指定指纹，如果为 None 则重置所有
        """
        with self._lock:
            if fingerprint:
                self._states.pop(fingerprint, None)
            else:
                self._states.clear()
    
    def cleanup_expired(self) -> int:
        """清理过期的状态。
        
        返回清理的状态数量。
        """
        now = datetime.now()
        expire_time = now - timedelta(seconds=self.window_seconds + self.suppress_seconds)
        
        cleaned = 0
        with self._lock:
            to_remove = []
            for fp, state in self._states.items():
                # 如果窗口为空且已过抑制期，可以清理
                if not state.event_timestamps:
                    if not state.last_alert_time or state.last_alert_time < expire_time:
                        to_remove.append(fp)
            
            for fp in to_remove:
                del self._states[fp]
                cleaned += 1
        
        return cleaned


__all__ = [
    "AggregationState",
    "AlertAggregator",
]



"""包管理命令。

用法：
    aury pkg list                    # 列出所有可安装模块
    aury pkg list --installed        # 已安装的模块
    aury pkg preset                  # 列出预设
    aury pkg preset api              # 查看预设详情
    aury pkg install postgres redis  # 安装模块
    aury pkg install --preset api    # 按预设安装
    aury pkg remove redis            # 卸载模块
"""

from __future__ import annotations

from dataclasses import dataclass
from enum import Enum
import subprocess
import sys
from typing import Annotated

from rich.console import Console
from rich.table import Table
import typer

console = Console()

app = typer.Typer(
    name="pkg",
    help="包管理 - 安装/卸载 aury 生态模块",
    no_args_is_help=True,
)


# ============================================================================
# 数据定义
# ============================================================================


class Category(str, Enum):
    """模块分类。"""

    DATABASE = "database"
    CACHE = "cache"
    TASK = "task"
    SCHEDULER = "scheduler"
    ADMIN = "admin"
    STORAGE = "storage"
    MONITORING = "monitoring"
    ECOSYSTEM = "ecosystem"


@dataclass
class ModuleInfo:
    """模块信息。"""

    name: str
    desc: str
    usage: str
    category: Category
    deps: list[str]  # extras 的依赖包名
    is_extra: bool = True  # True=extras, False=生态包
    pkg: str | None = None  # 生态包的完整包名


# Extras（aury-boot 的可选依赖）
MODULES: dict[str, ModuleInfo] = {
    # 数据库驱动
    "postgres": ModuleInfo(
        name="postgres",
        desc="PostgreSQL 异步驱动",
        usage="DatabaseManager 使用 PostgreSQL 时需要",
        category=Category.DATABASE,
        deps=["asyncpg"],
    ),
    "mysql": ModuleInfo(
        name="mysql",
        desc="MySQL 异步驱动",
        usage="DatabaseManager 使用 MySQL 时需要",
        category=Category.DATABASE,
        deps=["aiomysql"],
    ),
    "sqlite": ModuleInfo(
        name="sqlite",
        desc="SQLite 异步驱动",
        usage="DatabaseManager 使用 SQLite 时需要（本地开发推荐）",
        category=Category.DATABASE,
        deps=["aiosqlite"],
    ),
    # 缓存
    "redis": ModuleInfo(
        name="redis",
        desc="Redis 客户端",
        usage="CacheManager 使用 Redis 后端时需要",
        category=Category.CACHE,
        deps=["redis"],
    ),
    # 任务队列
    "tasks": ModuleInfo(
        name="tasks",
        desc="Dramatiq 任务队列",
        usage="TaskManager 异步任务时需要（默认使用 Redis Broker）",
        category=Category.TASK,
        deps=["dramatiq", "redis"],
    ),
    "rabbitmq": ModuleInfo(
        name="rabbitmq",
        desc="RabbitMQ 消息队列后端",
        usage="TaskManager/EventBus 使用 RabbitMQ 时需要（需配合 tasks）",
        category=Category.TASK,
        deps=["pika"],
    ),
    # 调度器
    "scheduler": ModuleInfo(
        name="scheduler",
        desc="APScheduler 定时调度",
        usage="SchedulerManager 定时任务时需要",
        category=Category.SCHEDULER,
        deps=["apscheduler"],
    ),
    # 管理后台
    "admin": ModuleInfo(
        name="admin",
        desc="SQLAdmin 管理后台",
        usage="启用 /admin 管理界面时需要",
        category=Category.ADMIN,
        deps=["sqladmin", "itsdangerous"],
    ),
    # 存储（extras）
    "s3": ModuleInfo(
        name="s3",
        desc="S3 兼容存储（AWS/MinIO/OSS）",
        usage="StorageManager 使用 S3 兼容存储时需要",
        category=Category.STORAGE,
        deps=["aury-sdk-storage[aws]"],
    ),
    # 监控
    "otel": ModuleInfo(
        name="otel",
        desc="OpenTelemetry 链路追踪",
        usage="启用 TELEMETRY__ENABLED 自动 instrument FastAPI/SQLAlchemy/httpx",
        category=Category.MONITORING,
        deps=["opentelemetry-api", "opentelemetry-sdk", "opentelemetry-instrumentation-fastapi",
              "opentelemetry-instrumentation-sqlalchemy", "opentelemetry-instrumentation-httpx"],
    ),
    "otel-exporter": ModuleInfo(
        name="otel-exporter",
        desc="OpenTelemetry OTLP 导出器",
        usage="导出 Traces/Metrics/Logs 到 Jaeger/Prometheus/Loki",
        category=Category.MONITORING,
        deps=["opentelemetry-exporter-otlp"],
    ),
    "profiling": ModuleInfo(
        name="profiling",
        desc="Profiling 性能分析",
        usage="Pyroscope 火焰图 + 事件循环阻塞检测",
        category=Category.MONITORING,
        deps=["pyroscope-io", "psutil"],
    ),
    # 生态包
    "storage-aws": ModuleInfo(
        name="storage-aws",
        desc="AWS S3 兼容存储",
        usage="StorageManager 使用 AWS S3/MinIO/OSS 时需要",
        category=Category.ECOSYSTEM,
        deps=[],
        is_extra=False,
        pkg="aury-sdk-storage[aws]",
    ),
    "storage-cos": ModuleInfo(
        name="storage-cos",
        desc="腾讯云 COS 原生存储",
        usage="StorageManager 使用腾讯云 COS 时推荐（性能更好）",
        category=Category.ECOSYSTEM,
        deps=[],
        is_extra=False,
        pkg="aury-sdk-storage[cos]",
    ),
}


@dataclass
class PresetInfo:
    """预设信息。"""

    name: str
    desc: str
    modules: list[str]


PRESETS: dict[str, PresetInfo] = {
    "minimal": PresetInfo(
        name="minimal",
        desc="最小化（本地开发/测试）",
        modules=["sqlite"],
    ),
    "api": PresetInfo(
        name="api",
        desc="API 服务（Web 接口 + 管理后台）",
        modules=["postgres", "redis", "admin"],
    ),
    "worker": PresetInfo(
        name="worker",
        desc="后台 Worker（任务队列 + 调度器）",
        modules=["postgres", "redis", "tasks", "rabbitmq", "scheduler"],
    ),
    "full": PresetInfo(
        name="full",
        desc="完整功能（所有模块）",
        modules=["postgres", "redis", "tasks", "rabbitmq", "scheduler", "admin", "storage-cos", "otel", "profiling"],
    ),
    "monitoring": PresetInfo(
        name="monitoring",
        desc="完整监控（OTel + Profiling）",
        modules=["otel", "otel-exporter", "profiling"],
    ),
}


# 分类显示名称
CATEGORY_NAMES: dict[Category, str] = {
    Category.DATABASE: "📦 数据库驱动",
    Category.CACHE: "📦 缓存",
    Category.TASK: "📦 任务队列",
    Category.SCHEDULER: "📦 定时调度",
    Category.ADMIN: "📦 管理后台",
    Category.STORAGE: "📦 对象存储",
    Category.MONITORING: "📊 监控分析",
    Category.ECOSYSTEM: "🌐 生态包",
}


# ============================================================================
# 工具函数
# ============================================================================


def _get_installed_packages() -> set[str]:
    """获取已安装的包名。"""
    try:
        result = subprocess.run(
            [sys.executable, "-m", "pip", "list", "--format=freeze"],
            capture_output=True,
            text=True,
            check=True,
        )
        installed = set()
        for line in result.stdout.strip().split("\n"):
            if "==" in line:
                pkg_name = line.split("==")[0].lower().replace("-", "_")
                installed.add(pkg_name)
        return installed
    except subprocess.CalledProcessError:
        return set()


def _is_module_installed(module: ModuleInfo) -> bool:
    """检查模块是否已安装。"""
    installed = _get_installed_packages()

    if module.is_extra:
        # 检查 deps 中的包是否已安装
        for dep in module.deps:
            # 处理 extras 语法，如 aury-sdk-storage[aws]
            pkg_name = dep.split("[")[0].lower().replace("-", "_")
            if pkg_name not in installed:
                return False
        return bool(module.deps)
    else:
        # 生态包：检查包名
        if module.pkg:
            pkg_name = module.pkg.split("[")[0].lower().replace("-", "_")
            return pkg_name in installed
    return False


def _run_uv_command(args: list[str]) -> bool:
    """运行 uv 命令。"""
    cmd = ["uv", *args]
    console.print(f"[dim]$ {' '.join(cmd)}[/dim]")
    try:
        subprocess.run(cmd, check=True)
        return True
    except subprocess.CalledProcessError as e:
        console.print(f"[red]命令失败: {e}[/red]")
        return False
    except FileNotFoundError:
        console.print("[red]错误: 未找到 uv，请先安装: pip install uv[/red]")
        return False


# ============================================================================
# 命令实现
# ============================================================================


@app.command(name="list")
def list_modules(
    installed: Annotated[
        bool,
        typer.Option("--installed", "-i", help="仅显示已安装的模块"),
    ] = False,
) -> None:
    """列出所有可安装的模块。"""
    installed_pkgs = _get_installed_packages() if installed else None

    # 按分类组织
    by_category: dict[Category, list[ModuleInfo]] = {}
    for module in MODULES.values():
        if module.category not in by_category:
            by_category[module.category] = []
        by_category[module.category].append(module)

    # 输出
    for category in Category:
        modules = by_category.get(category, [])
        if not modules:
            continue

        # 过滤已安装
        if installed:
            modules = [m for m in modules if _is_module_installed(m)]
            if not modules:
                continue

        console.print()
        console.print(f"[bold]{CATEGORY_NAMES[category]}[/bold]")

        table = Table(show_header=False, box=None, padding=(0, 2))
        table.add_column("名称", style="cyan", width=15)
        table.add_column("描述", width=30)
        table.add_column("用途", style="dim")

        for module in modules:
            status = ""
            if installed_pkgs is not None:
                is_installed = _is_module_installed(module)
                status = " [green]✓[/green]" if is_installed else ""
            table.add_row(
                f"{module.name}{status}",
                module.desc,
                f"→ {module.usage}",
            )

        console.print(table)

    console.print()
    console.print("[dim]提示: 使用 aury pkg install <模块名> 安装模块[/dim]")


@app.command(name="preset")
def list_presets(
    name: Annotated[
        str | None,
        typer.Argument(help="预设名称（留空列出所有预设）"),
    ] = None,
) -> None:
    """查看预设配置。"""
    if name is None:
        # 列出所有预设
        console.print()
        console.print("[bold]📋 可用预设[/bold]")
        console.print()

        table = Table(show_header=True, box=None, padding=(0, 2))
        table.add_column("预设", style="cyan", width=12)
        table.add_column("描述", width=35)
        table.add_column("包含模块", style="dim")

        for preset in PRESETS.values():
            table.add_row(
                preset.name,
                preset.desc,
                ", ".join(preset.modules),
            )

        console.print(table)
        console.print()
        console.print("[dim]提示: 使用 aury pkg preset <预设名> 查看详情[/dim]")
        console.print("[dim]      使用 aury pkg install --preset <预设名> 安装[/dim]")
    else:
        # 查看指定预设
        if name not in PRESETS:
            console.print(f"[red]错误: 预设 '{name}' 不存在[/red]")
            console.print(f"[dim]可用预设: {', '.join(PRESETS.keys())}[/dim]")
            raise typer.Exit(1)

        preset = PRESETS[name]
        console.print()
        console.print(f"[bold]📋 预设: {preset.name}[/bold]")
        console.print(f"[dim]{preset.desc}[/dim]")
        console.print()

        console.print("[bold]包含模块:[/bold]")
        for module_name in preset.modules:
            module = MODULES.get(module_name)
            if module:
                installed = _is_module_installed(module)
                status = "[green]✓ 已安装[/green]" if installed else "[dim]未安装[/dim]"
                console.print(f"  • {module.name}: {module.desc} {status}")
            else:
                console.print(f"  • {module_name} [red](未知模块)[/red]")

        console.print()
        console.print(f"[dim]安装命令: aury pkg install --preset {name}[/dim]")


@app.command(name="install")
def install_modules(
    modules: Annotated[
        list[str] | None,
        typer.Argument(help="要安装的模块名称"),
    ] = None,
    preset: Annotated[
        str | None,
        typer.Option("--preset", "-p", help="使用预设安装"),
    ] = None,
) -> None:
    """安装模块。"""
    if preset:
        # 使用预设
        if preset not in PRESETS:
            console.print(f"[red]错误: 预设 '{preset}' 不存在[/red]")
            console.print(f"[dim]可用预设: {', '.join(PRESETS.keys())}[/dim]")
            raise typer.Exit(1)

        preset_info = PRESETS[preset]
        modules = preset_info.modules
        console.print(f"[bold]📦 安装预设: {preset_info.name}[/bold]")
        console.print(f"[dim]{preset_info.desc}[/dim]")
        console.print()

    if not modules:
        console.print("[red]错误: 请指定要安装的模块，或使用 --preset[/red]")
        raise typer.Exit(1)

    # 收集要安装的包
    extras_to_install: list[str] = []
    pkgs_to_install: list[str] = []

    for module_name in modules:
        if module_name not in MODULES:
            console.print(f"[yellow]警告: 模块 '{module_name}' 不存在，跳过[/yellow]")
            continue

        module = MODULES[module_name]
        if module.is_extra:
            extras_to_install.append(module.name)
        else:
            if module.pkg:
                pkgs_to_install.append(module.pkg)

    # 安装 extras
    if extras_to_install:
        extras_str = ",".join(extras_to_install)
        console.print(f"[bold]安装 extras: {extras_str}[/bold]")
        if not _run_uv_command(["add", f"aury-boot[{extras_str}]"]):
            raise typer.Exit(1)

    # 安装生态包
    for pkg in pkgs_to_install:
        console.print(f"[bold]安装生态包: {pkg}[/bold]")
        if not _run_uv_command(["add", pkg]):
            raise typer.Exit(1)

    console.print()
    console.print("[green]✅ 安装完成[/green]")


@app.command(name="remove")
def remove_modules(
    modules: Annotated[
        list[str],
        typer.Argument(help="要卸载的模块名称"),
    ],
) -> None:
    """卸载模块。"""
    for module_name in modules:
        if module_name not in MODULES:
            console.print(f"[yellow]警告: 模块 '{module_name}' 不存在，跳过[/yellow]")
            continue

        module = MODULES[module_name]

        if module.is_extra:
            # extras 需要移除具体的依赖包
            console.print(f"[bold]移除 {module.name} 的依赖...[/bold]")
            for dep in module.deps:
                pkg_name = dep.split("[")[0]  # 去掉 extras 语法
                _run_uv_command(["remove", pkg_name])
        else:
            # 生态包直接移除
            if module.pkg:
                pkg_name = module.pkg.split("[")[0]
                console.print(f"[bold]移除生态包: {pkg_name}[/bold]")
                _run_uv_command(["remove", pkg_name])

    console.print()
    console.print("[green]✅ 卸载完成[/green]")


__all__ = ["app"]



"""健康检查模块（待实现）。

提供可插拔的健康检查功能，支持 Kubernetes 标准探针。

TODO: 实现以下功能
"""

from __future__ import annotations

# =============================================================================
# 以下为伪代码，待实现
# =============================================================================

# from abc import ABC, abstractmethod
# from enum import Enum
# from typing import Literal
# import asyncio
# from dataclasses import dataclass, field
#
#
# class ProbeType(str, Enum):
#     """探针类型。"""
#     LIVENESS = "liveness"    # 存活检查，失败会重启
#     READINESS = "readiness"  # 就绪检查，失败从负载均衡移除
#     STARTUP = "startup"      # 启动检查
#
#
# @dataclass
# class HealthCheck:
#     """健康检查项。"""
#     name: str
#     check_func: Callable[[], Awaitable[None]]
#     timeout: float = 5.0
#     critical: bool = True  # 失败是否导致整体失败
#
#     async def run(self) -> tuple[str, bool, str | None]:
#         """执行检查。
#
#         Returns:
#             (name, success, error_message)
#         """
#         try:
#             await asyncio.wait_for(self.check_func(), timeout=self.timeout)
#             return (self.name, True, None)
#         except asyncio.TimeoutError:
#             return (self.name, False, f"Timeout after {self.timeout}s")
#         except Exception as e:
#             return (self.name, False, str(e))
#
#
# @dataclass
# class HealthResult:
#     """健康检查结果。"""
#     status: Literal["healthy", "unhealthy", "degraded"]
#     checks: dict[str, dict] = field(default_factory=dict)
#
#
# class HealthManager:
#     """可插拔健康检查管理器。"""
#
#     _checks: dict[str, list[HealthCheck]] = {
#         "liveness": [],
#         "readiness": [],
#         "startup": [],
#     }
#
#     @classmethod
#     def register(
#         cls,
#         name: str,
#         probe: Literal["liveness", "readiness", "startup"] = "readiness",
#         timeout: float = 5.0,
#         critical: bool = True,
#     ):
#         """装饰器注册检查器。
#
#         用法:
#             @HealthManager.register("database", probe="readiness")
#             async def check_db():
#                 await db.execute("SELECT 1")
#
#             @HealthManager.register("redis", probe="readiness", critical=False)
#             async def check_redis():
#                 await redis.ping()
#
#             @HealthManager.register("app", probe="liveness")
#             async def check_app():
#                 return True
#         """
#         def decorator(func):
#             cls._checks[probe].append(
#                 HealthCheck(name, func, timeout, critical)
#             )
#             return func
#         return decorator
#
#     @classmethod
#     async def check(
#         cls,
#         probe: str,
#         detailed: bool = False,
#     ) -> HealthResult:
#         """执行指定探针的所有检查（并行+超时）。
#
#         Args:
#             probe: 探针类型 (liveness/readiness/startup)
#             detailed: 是否返回详细信息
#
#         Returns:
#             HealthResult: 检查结果
#         """
#         checks = cls._checks.get(probe, [])
#         if not checks:
#             return HealthResult(status="healthy")
#
#         # 并行执行所有检查
#         results = await asyncio.gather(
#             *[c.run() for c in checks],
#             return_exceptions=True
#         )
#
#         # 汇总结果
#         all_ok = True
#         has_degraded = False
#         check_results = {}
#
#         for check, result in zip(checks, results):
#             if isinstance(result, Exception):
#                 name, success, error = check.name, False, str(result)
#             else:
#                 name, success, error = result
#
#             check_results[name] = {
#                 "status": "ok" if success else "error",
#                 "error": error,
#             }
#
#             if not success:
#                 if check.critical:
#                     all_ok = False
#                 else:
#                     has_degraded = True
#
#         if all_ok:
#             status = "degraded" if has_degraded else "healthy"
#         else:
#             status = "unhealthy"
#
#         return HealthResult(
#             status=status,
#             checks=check_results if detailed else {},
#         )
#
#     @classmethod
#     def clear(cls) -> None:
#         """清除所有注册的检查器。"""
#         for probe in cls._checks:
#             cls._checks[probe] = []
#
#
# # =============================================================================
# # 内置检查器
# # =============================================================================
#
#
# def register_database_check(db_manager) -> None:
#     """注册数据库健康检查。"""
#     @HealthManager.register("database", probe="readiness")
#     async def check_database():
#         await db_manager.health_check()
#
#
# def register_cache_check(cache_manager) -> None:
#     """注册缓存健康检查。"""
#     @HealthManager.register("cache", probe="readiness", critical=False)
#     async def check_cache():
#         await cache_manager.get("__health__", default=None)
#
#
# def register_redis_check(redis_client) -> None:
#     """注册 Redis 健康检查。"""
#     @HealthManager.register("redis", probe="readiness", critical=False)
#     async def check_redis():
#         await redis_client.ping()
#
#
# # =============================================================================
# # 路由注册（在 FoundationApp 中调用）
# # =============================================================================
#
#
# def setup_health_routes(app) -> None:
#     """注册健康检查路由。
#
#     路由:
#         GET /health/live   -> liveness checks
#         GET /health/ready  -> readiness checks
#         GET /health/startup -> startup checks
#     """
#     from fastapi import status
#     from fastapi.responses import JSONResponse
#
#     @app.get("/health/live", tags=["health"])
#     async def liveness():
#         result = await HealthManager.check("liveness")
#         return JSONResponse(
#             content={"status": result.status},
#             status_code=status.HTTP_200_OK if result.status == "healthy"
#                 else status.HTTP_503_SERVICE_UNAVAILABLE,
#         )
#
#     @app.get("/health/ready", tags=["health"])
#     async def readiness():
#         result = await HealthManager.check("readiness", detailed=True)
#         return JSONResponse(
#             content={"status": result.status, "checks": result.checks},
#             status_code=status.HTTP_200_OK if result.status != "unhealthy"
#                 else status.HTTP_503_SERVICE_UNAVAILABLE,
#         )
#
#     @app.get("/health/startup", tags=["health"])
#     async def startup():
#         result = await HealthManager.check("startup")
#         return JSONResponse(
#             content={"status": result.status},
#             status_code=status.HTTP_200_OK if result.status == "healthy"
#                 else status.HTTP_503_SERVICE_UNAVAILABLE,
#         )


__all__: list[str] = []



"""缓存管理器 - 命名多实例模式。

提供统一的缓存管理接口。
"""

from __future__ import annotations

import asyncio
from collections.abc import Callable
from contextlib import asynccontextmanager
from datetime import timedelta
from functools import wraps
import hashlib
import time
from typing import Any, AsyncIterator, TypeVar
import uuid

from aury.boot.common.logging import logger

# from aury.boot.config import settings  # TODO: 需要从应用配置中获取
from .base import CacheBackend, ICache
from .factory import CacheFactory


class CacheManager:
    """缓存管理器（命名多实例）。
    
    支持多个命名实例，如不同的 Redis 实例或缓存策略。
    
    使用示例:
        # 默认实例
        cache = CacheManager.get_instance()
        await cache.initialize(backend="redis", url="redis://localhost:6379")
        
        # 命名实例
        session_cache = CacheManager.get_instance("session")
        rate_limit_cache = CacheManager.get_instance("rate_limit")
        
        # 使用
        await cache.set("key", "value", expire=60)
        value = await cache.get("key")
    """
    
    _instances: dict[str, CacheManager] = {}
    
    def __init__(self, name: str = "default") -> None:
        """初始化缓存管理器。
        
        Args:
            name: 实例名称
        """
        self.name = name
        self._backend: ICache | None = None
        self._config: dict[str, Any] = {}
    
    @classmethod
    def get_instance(cls, name: str = "default") -> CacheManager:
        """获取指定名称的实例。
        
        Args:
            name: 实例名称，默认为 "default"
            
        Returns:
            CacheManager: 缓存管理器实例
        """
        if name not in cls._instances:
            cls._instances[name] = cls(name)
        return cls._instances[name]
    
    @classmethod
    def reset_instance(cls, name: str | None = None) -> None:
        """重置实例（仅用于测试）。
        
        Args:
            name: 要重置的实例名称。如果为 None，则重置所有实例。
            
        注意：调用此方法前应先调用 cleanup() 释放资源。
        """
        if name is None:
            cls._instances.clear()
        elif name in cls._instances:
            del cls._instances[name]
    
    async def initialize(
        self,
        backend: CacheBackend | str = CacheBackend.REDIS,
        *,
        url: str | None = None,
        max_size: int = 1000,
        serializer: str = "json",
        servers: list[str] | None = None,
    ) -> CacheManager:
        """初始化缓存（链式调用）。
        
        Args:
            backend: 缓存后端类型（redis/memory/memcached）
            url: Redis/Memcached 连接 URL
            max_size: 内存缓存最大容量（仅 memory 后端）
            serializer: 序列化方式（json/pickle）
            servers: Memcached 服务器列表（已弃用，请使用 url）
            
        Returns:
            self: 支持链式调用
        """
        if self._backend is not None:
            logger.warning(f"缓存管理器 [{self.name}] 已初始化，跳过")
            return self
        
        # 处理字符串类型的 backend
        if isinstance(backend, str):
            try:
                backend = CacheBackend(backend.lower())
            except ValueError:
                supported = ", ".join(b.value for b in CacheBackend)
                raise ValueError(f"不支持的缓存后端: {backend}。支持: {supported}")
        
        # 保存配置（用于启动横幅等场景展示）
        self._config = {"CACHE_TYPE": backend.value}
        
        # 根据后端类型构建配置并创建后端
        if backend == CacheBackend.REDIS:
            if not url:
                raise ValueError("Redis 缓存需要提供 url 参数")
            # 记录 URL 以便在启动横幅中展示（会通过 mask_url 脱敏）
            self._config["CACHE_URL"] = url
            self._backend = await CacheFactory.create(
                "redis", url=url, serializer=serializer
            )
        elif backend == CacheBackend.MEMORY:
            self._backend = await CacheFactory.create(
                "memory", max_size=max_size
            )
        elif backend == CacheBackend.MEMCACHED:
            cache_url = url or (servers[0] if servers else None)
            if not cache_url:
                raise ValueError("Memcached 缓存需要提供 url 参数")
            # 同样记录 URL，便于在启动横幅中展示
            self._config["CACHE_URL"] = cache_url
            self._backend = await CacheFactory.create(
                "memcached", servers=cache_url
            )
        else:
            supported = ", ".join(b.value for b in CacheBackend)
            raise ValueError(f"不支持的缓存后端: {backend}。支持: {supported}")
        
        logger.info(f"缓存管理器 [{self.name}] 初始化完成: {backend.value}")
        return self
    
    @property
    def is_initialized(self) -> bool:
        """检查是否已初始化。"""
        return self._backend is not None
    
    @property
    def backend(self) -> ICache:
        """获取缓存后端。"""
        if self._backend is None:
            raise RuntimeError("缓存管理器未初始化，请先调用 initialize()")
        return self._backend
    
    @property
    def backend_type(self) -> str:
        """获取当前使用的后端类型。"""
        return self._config.get("CACHE_TYPE", "unknown")
    
    async def get(self, key: str, default: Any = None) -> Any:
        """获取缓存。"""
        return await self.backend.get(key, default)
    
    async def set(
        self,
        key: str,
        value: Any,
        expire: int | timedelta | None = None,
    ) -> bool:
        """设置缓存。"""
        return await self.backend.set(key, value, expire)
    
    async def delete(self, *keys: str) -> int:
        """删除缓存。"""
        return await self.backend.delete(*keys)
    
    async def exists(self, *keys: str) -> int:
        """检查缓存是否存在。"""
        return await self.backend.exists(*keys)
    
    async def clear(self) -> None:
        """清空所有缓存。"""
        await self.backend.clear()
    
    async def delete_pattern(self, pattern: str) -> int:
        """按模式删除缓存。
        
        Args:
            pattern: 通配符模式，如 "todo:*" 或 "api:todo:list:*"
            
        Returns:
            int: 删除的键数量
            
        示例:
            # 删除所有 todo 相关缓存
            await cache.delete_pattern("todo:*")
            
            # 删除所有列表缓存
            await cache.delete_pattern("api:todo:list:*")
        """
        return await self.backend.delete_pattern(pattern)
    
    def cached[T](
        self,
        expire: int | timedelta | None = None,
        *,
        key_prefix: str = "",
    ) -> Callable[[Callable[..., T]], Callable[..., T]]:
        """缓存装饰器。
        
        Args:
            expire: 过期时间
            key_prefix: 键前缀
        """
        def decorator(func: Callable[..., T]) -> Callable[..., T]:
            @wraps(func)
            async def wrapper(*args, **kwargs) -> T:
                # 生成缓存键
                func_name = f"{func.__module__}.{func.__name__}"
                args_str = str(args) + str(sorted(kwargs.items()))
                key_hash = hashlib.md5(args_str.encode()).hexdigest()[:8]
                cache_key = f"{key_prefix}:{func_name}:{key_hash}" if key_prefix else f"{func_name}:{key_hash}"
                
                # 尝试获取缓存
                cached_value = await self.get(cache_key)
                if cached_value is not None:
                    logger.debug(f"缓存命中: {cache_key}")
                    return cached_value
                
                # 执行函数
                result = await func(*args, **kwargs)
                
                # 存入缓存
                await self.set(cache_key, result, expire)
                logger.debug(f"缓存更新: {cache_key}")
                
                return result
            
            return wrapper
        return decorator
    
    def cache_response[T](
        self,
        expire: int | timedelta | None = 300,
        *,
        key_builder: Callable[..., str] | None = None,
        key_prefix: str = "api",
    ) -> Callable[[Callable[..., T]], Callable[..., T]]:
        """API 响应缓存装饰器。
        
        专为 FastAPI 路由设计，自动从路径参数和查询参数生成缓存键。
        
        Args:
            expire: 过期时间（秒），默认 300 秒
            key_builder: 自定义缓存键生成函数，接收与被装饰函数相同的参数
            key_prefix: 缓存键前缀，默认 "api"
            
        示例:
            cache = CacheManager.get_instance()
            
            # 基本用法：自动生成缓存键
            @router.get("/todos/{{id}}")
            @cache.cache_response(expire=300)
            async def get_todo(id: UUID):
                return await service.get(id)
            # 缓存键: api:get_todo:<hash>
            
            # 自定义缓存键
            @router.get("/todos/{{id}}")
            @cache.cache_response(
                expire=300,
                key_builder=lambda id: f"todo:{{id}}"
            )
            async def get_todo(id: UUID):
                return await service.get(id)
            # 缓存键: api:todo:<id>
        """
        def decorator(func: Callable[..., T]) -> Callable[..., T]:
            @wraps(func)
            async def wrapper(*args, **kwargs) -> T:
                # 生成缓存键
                if key_builder:
                    # 使用自定义的 key_builder
                    custom_key = key_builder(*args, **kwargs)
                    cache_key = f"{key_prefix}:{custom_key}" if key_prefix else custom_key
                else:
                    # 自动生成：函数名 + 参数哈希
                    func_name = func.__name__
                    args_str = str(args) + str(sorted(kwargs.items()))
                    key_hash = hashlib.md5(args_str.encode()).hexdigest()[:8]
                    cache_key = f"{key_prefix}:{func_name}:{key_hash}"
                
                # 尝试从缓存获取
                cached_value = await self.get(cache_key)
                if cached_value is not None:
                    logger.debug(f"API 缓存命中: {cache_key}")
                    return cached_value
                
                # 执行函数
                result = await func(*args, **kwargs)
                
                # 存入缓存（尝试序列化）
                try:
                    # 如果结果有 model_dump 方法（Pydantic model），先序列化
                    if hasattr(result, "model_dump"):
                        cache_data = result.model_dump()
                    elif hasattr(result, "dict"):
                        cache_data = result.dict()
                    else:
                        cache_data = result
                    
                    await self.set(cache_key, cache_data, expire)
                    logger.debug(f"API 缓存更新: {cache_key}")
                except Exception as e:
                    logger.warning(f"API 缓存存储失败: {cache_key}, {e}")
                
                return result
            
            return wrapper
        return decorator
    
    async def cleanup(self) -> None:
        """清理资源。"""
        if self._backend:
            await self._backend.close()
            self._backend = None
            logger.info("缓存管理器已清理")
    
    # ==================== 分布式锁 ====================
    
    async def acquire_lock(
        self,
        key: str,
        *,
        timeout: int = 30,
        blocking: bool = True,
        blocking_timeout: float | None = None,
    ) -> str | None:
        """获取分布式锁。
        
        Args:
            key: 锁的键名
            timeout: 锁的超时时间（秒），防止死锁
            blocking: 是否阻塞等待
            blocking_timeout: 阻塞等待的最大时间（秒）
            
        Returns:
            str | None: 锁的 token（用于释放），获取失败返回 None
        """
        lock_key = f"lock:{key}"
        token = str(uuid.uuid4())
        
        acquired = await self.backend.acquire_lock(
            lock_key, token, timeout, blocking, blocking_timeout
        )
        return token if acquired else None
    
    async def release_lock(self, key: str, token: str) -> bool:
        """释放分布式锁。
        
        Args:
            key: 锁的键名
            token: acquire_lock 返回的 token
            
        Returns:
            bool: 是否成功释放
        """
        lock_key = f"lock:{key}"
        return await self.backend.release_lock(lock_key, token)
    
    @asynccontextmanager
    async def lock(
        self,
        key: str,
        *,
        timeout: int = 30,
        blocking: bool = True,
        blocking_timeout: float | None = None,
    ) -> AsyncIterator[bool]:
        """分布式锁上下文管理器。
        
        Args:
            key: 锁的键名
            timeout: 锁的超时时间（秒）
            blocking: 是否阻塞等待
            blocking_timeout: 阻塞等待的最大时间（秒）
            
        Yields:
            bool: 是否成功获取锁
            
        示例:
            async with cache.lock("my_resource") as acquired:
                if acquired:
                    # 执行需要互斥的操作
                    pass
        """
        token = await self.acquire_lock(
            key,
            timeout=timeout,
            blocking=blocking,
            blocking_timeout=blocking_timeout,
        )
        try:
            yield token is not None
        finally:
            if token:
                await self.release_lock(key, token)
    
    @asynccontextmanager
    async def semaphore(
        self,
        key: str,
        max_concurrency: int,
        *,
        timeout: int = 300,
        blocking: bool = True,
        blocking_timeout: float | None = None,
    ) -> AsyncIterator[bool]:
        """分布式信号量（限制并发数）。
        
        Args:
            key: 信号量的键名
            max_concurrency: 最大并发数
            timeout: 单个槽位的超时时间（秒）
            blocking: 是否阻塞等待
            blocking_timeout: 阻塞等待的最大时间（秒）
            
        Yields:
            bool: 是否成功获取槽位
            
        示例:
            async with cache.semaphore("pdf_ocr", max_concurrency=2) as acquired:
                if acquired:
                    # 执行受并发限制的操作
                    pass
        """
        slot_token: str | None = None
        acquired_slot: int | None = None
        start_time = time.monotonic()
        
        try:
            while True:
                # 尝试获取任意一个槽位
                for slot in range(max_concurrency):
                    slot_key = f"{key}:slot:{slot}"
                    token = await self.acquire_lock(
                        slot_key,
                        timeout=timeout,
                        blocking=False,
                    )
                    if token:
                        slot_token = token
                        acquired_slot = slot
                        yield True
                        return
                
                if not blocking:
                    yield False
                    return
                
                # 检查是否超时
                if blocking_timeout is not None:
                    elapsed = time.monotonic() - start_time
                    if elapsed >= blocking_timeout:
                        yield False
                        return
                
                # 等待后重试
                await asyncio.sleep(0.1)
        finally:
            if slot_token and acquired_slot is not None:
                slot_key = f"{key}:slot:{acquired_slot}"
                await self.release_lock(slot_key, slot_token)
    
    def __repr__(self) -> str:
        """字符串表示。"""
        backend_name = self.backend_type if self._backend else "未初始化"
        return f"<CacheManager backend={backend_name}>"


__all__ = [
    "CacheManager",
]




"""第三方接口适配器模块。

本模块用于封装第三方接口（如支付、短信、微信、云存储等外部服务）的调用，
解决测试环境中第三方接口不可用的问题，通过配置切换真实调用与挡板（Mock）实现。

核心特性:
- 多模式切换：real(真实)/sandbox(沙箱)/mock(挡板)/disabled(禁用)
- 按方法级别的模式覆盖：同一 Adapter 不同方法可用不同模式
- 装饰器声明式定义：@adapter_method + .mock 链式定义挡板逻辑
- HTTP 类第三方的便捷基类：HttpAdapter 封装请求/认证/错误处理
- 调用历史记录：便于测试断言，验证调用参数和结果

模式说明:
- real: 调用真实第三方接口（生产环境）
- sandbox: 调用第三方提供的沙箱环境（如果有）
- mock: 使用本地挡板实现，不发出真实请求（测试/开发环境）
- disabled: 禁用该接口，调用时抛出 AdapterDisabledError

设计原则:
- 最小约束：只约束"模式怎么切换"，不限制具体调用方式
- 最大自由：HTTP/SDK/gRPC/任意协议都能用
- 可组合：Adapter 可以组合其他 Adapter

典型使用场景:
1. 测试环境：第三方接口不可用，使用 mock 模式运行测试
2. 开发环境：避免消耗真实资源（如短信费用），使用 mock
3. 部分挡板：查询接口走真实，写入接口走 mock
4. 功能开关：禁用某些危险操作（如退款）在测试环境

使用示例:
    from aury.boot.application.adapter import (
        HttpAdapter, adapter_method, AdapterSettings
    )
    
    class PaymentAdapter(HttpAdapter):
        '''支付第三方接口适配器。'''
        
        @adapter_method("create")
        async def create_order(self, amount: int, order_id: str) -> dict:
            '''创建支付订单（真实实现）。'''
            return await self._request("POST", "/charge", json={"amount": amount, "order_id": order_id})
        
        @create_order.mock
        async def create_order_mock(self, amount: int, order_id: str) -> dict:
            '''创建支付订单（挡板实现）。
            
            挡板可以包含业务逻辑，模拟各种场景（成功、失败、边界情况）。
            '''
            if amount > 10000:
                return {"success": False, "error": "金额超限"}
            return {"success": True, "mock": True, "order_id": order_id}
    
    # 配置：测试环境使用挡板，但 query 走真实接口
    settings = AdapterSettings(
        mode="mock",                      # 默认走挡板
        method_modes={"query": "real"},   # query 走真实
        base_url="https://api.payment.com",
    )
    
    adapter = PaymentAdapter("payment", settings)
    result = await adapter.create_order(100, "order-1")  # 走挡板

环境变量配置:
    # 全局模式
    THIRD_PARTY_PAYMENT_MODE=mock
    
    # 连接配置
    THIRD_PARTY_PAYMENT_BASE_URL=https://api.payment.com
    THIRD_PARTY_PAYMENT_SANDBOX_URL=https://sandbox.payment.com
    THIRD_PARTY_PAYMENT_API_KEY=sk_test_xxx
    THIRD_PARTY_PAYMENT_TIMEOUT=30
    
    # 方法级模式覆盖（JSON 格式）
    THIRD_PARTY_PAYMENT_METHOD_MODES={"query": "real", "refund": "disabled"}
    
    # 挡板配置
    THIRD_PARTY_PAYMENT_MOCK_DELAY=0.1  # 模拟网络延迟
"""

from .base import BaseAdapter, CallRecord
from .config import (
    AdapterMode,
    AdapterSettings,
    MockStrategy,
    ThirdPartySettings,
)
from .decorators import adapter_method
from .exceptions import (
    AdapterDisabledError,
    AdapterError,
    AdapterTimeoutError,
)
from .http import HttpAdapter

__all__ = [
    # 基类
    "BaseAdapter",
    "HttpAdapter",
    # 配置
    "AdapterSettings",
    "AdapterMode",
    "MockStrategy",
    "ThirdPartySettings",
    # 装饰器
    "adapter_method",
    # 异常
    "AdapterError",
    "AdapterDisabledError",
    "AdapterTimeoutError",
    # 数据类
    "CallRecord",
]



"""Redis 消息队列后端。

使用 Redis List 实现简单的消息队列。
"""

from __future__ import annotations

import asyncio
from collections.abc import Callable
import json
from typing import TYPE_CHECKING, Any

from aury.boot.common.logging import logger

from ..base import IMQ, MQMessage

if TYPE_CHECKING:
    from aury.boot.infrastructure.clients.redis import RedisClient


class RedisMQ(IMQ):
    """Redis 消息队列实现。

    使用 Redis List (LPUSH/BRPOP) 实现可靠的消息队列。
    """

    def __init__(
        self,
        url: str | None = None,
        *,
        redis_client: RedisClient | None = None,
        prefix: str = "mq:",
    ) -> None:
        """初始化 Redis 消息队列。

        Args:
            url: Redis 连接 URL（当 redis_client 为 None 时必须提供）
            redis_client: RedisClient 实例（可选，优先使用）
            prefix: 队列名称前缀
        
        Raises:
            ValueError: 当 url 和 redis_client 都为 None 时
        """
        if redis_client is None and url is None:
            raise ValueError("Redis 消息队列需要提供 url 或 redis_client 参数")
        
        self._url = url
        self._client = redis_client
        self._prefix = prefix
        self._consuming = False
        self._owns_client = False  # 是否自己创建的客户端
    
    async def _ensure_client(self) -> None:
        """确保 Redis 客户端已初始化。"""
        if self._client is None and self._url:
            from aury.boot.infrastructure.clients.redis import RedisClient
            self._client = RedisClient()
            await self._client.initialize(url=self._url)
            self._owns_client = True

    def _queue_key(self, queue: str) -> str:
        """获取队列的 Redis key。"""
        return f"{self._prefix}{queue}"

    def _processing_key(self, queue: str) -> str:
        """获取处理中队列的 Redis key。"""
        return f"{self._prefix}{queue}:processing"

    async def send(self, queue: str, message: MQMessage) -> str:
        """发送消息到队列。"""
        await self._ensure_client()
        message.queue = queue
        data = json.dumps(message.to_dict())
        await self._client.connection.lpush(self._queue_key(queue), data)
        return message.id

    async def receive(
        self,
        queue: str,
        timeout: float | None = None,
    ) -> MQMessage | None:
        """从队列接收消息。"""
        await self._ensure_client()
        timeout_int = int(timeout) if timeout else 0
        result = await self._client.connection.brpop(
            self._queue_key(queue),
            timeout=timeout_int,
        )
        if result is None:
            return None

        _, data = result
        try:
            msg_dict = json.loads(data)
            message = MQMessage.from_dict(msg_dict)
            # 将消息放入处理中队列
            await self._client.connection.hset(
                self._processing_key(queue),
                message.id,
                data,
            )
            return message
        except (json.JSONDecodeError, KeyError) as e:
            logger.error(f"解析消息失败: {e}")
            return None

    async def ack(self, message: MQMessage) -> None:
        """确认消息已处理。"""
        if message.queue:
            await self._client.connection.hdel(
                self._processing_key(message.queue),
                message.id,
            )

    async def nack(self, message: MQMessage, requeue: bool = True) -> None:
        """拒绝消息。"""
        if message.queue:
            # 从处理中队列移除
            await self._client.connection.hdel(
                self._processing_key(message.queue),
                message.id,
            )
            if requeue and message.retry_count < message.max_retries:
                # 重新入队
                message.retry_count += 1
                await self.send(message.queue, message)

    async def consume(
        self,
        queue: str,
        handler: Callable[[MQMessage], Any],
        *,
        prefetch: int = 1,
    ) -> None:
        """消费队列消息。"""
        self._consuming = True
        logger.info(f"开始消费队列: {queue}")

        while self._consuming:
            try:
                message = await self.receive(queue, timeout=1.0)
                if message is None:
                    continue

                try:
                    result = handler(message)
                    if asyncio.iscoroutine(result):
                        await result
                    await self.ack(message)
                except Exception as e:
                    logger.error(f"处理消息失败: {e}")
                    await self.nack(message, requeue=True)

            except Exception as e:
                logger.error(f"消费消息异常: {e}")
                await asyncio.sleep(1)

    async def close(self) -> None:
        """关闭连接。"""
        self._consuming = False
        if self._owns_client and self._client:
            await self._client.cleanup()
            self._client = None
        logger.debug("Redis 消息队列已关闭")


__all__ = ["RedisMQ"]



"""通道管理器 - 命名多实例模式。

提供统一的通道管理接口，支持多后端和多实例。
"""

from __future__ import annotations

from collections.abc import AsyncIterator
from urllib.parse import urlparse

from aury.boot.common.logging import logger

from .backends.broadcaster import BroadcasterChannel
from .backends.redis import RedisChannel
from .base import ChannelBackend, ChannelMessage, IChannel


class ChannelManager:
    """通道管理器（命名多实例）。

    提供统一的通道管理接口，支持：
    - 多实例管理（如 sse、notification 各自独立）
    - 多后端支持（memory、redis）
    - 发布/订阅模式

    使用示例:
        # 默认实例
        channel = ChannelManager.get_instance()
        await channel.initialize(backend="memory")

        # 命名实例
        sse_channel = ChannelManager.get_instance("sse")
        notification_channel = ChannelManager.get_instance("notification")

        # 发布消息
        await channel.publish("events", ChannelMessage(data="hello"))

        # 订阅消息
        async for msg in channel.subscribe("events"):
            print(msg.data)
    """

    _instances: dict[str, ChannelManager] = {}

    def __init__(self, name: str = "default") -> None:
        """初始化通道管理器。

        Args:
            name: 实例名称
        """
        self.name = name
        self._backend: IChannel | None = None
        self._backend_type: ChannelBackend | None = None
        self._initialized: bool = False
        self._url: str | None = None

    @classmethod
    def get_instance(cls, name: str = "default") -> ChannelManager:
        """获取指定名称的实例。

        Args:
            name: 实例名称，默认为 "default"

        Returns:
            ChannelManager: 通道管理器实例
        """
        if name not in cls._instances:
            cls._instances[name] = cls(name)
        return cls._instances[name]

    @classmethod
    def reset_instance(cls, name: str | None = None) -> None:
        """重置实例（仅用于测试）。

        Args:
            name: 要重置的实例名称。如果为 None，则重置所有实例。

        注意：调用此方法前应先调用 cleanup() 释放资源。
        """
        if name is None:
            cls._instances.clear()
        elif name in cls._instances:
            del cls._instances[name]

    async def initialize(
        self,
        backend: ChannelBackend | str = ChannelBackend.REDIS,
        *,
        url: str = "memory://",
    ) -> ChannelManager:
        """初始化通道（链式调用）。

        Args:
            backend: 后端类型，默认 redis（原生实现，解决并发问题）
            url: 连接 URL，支持：
                - memory:// - 内存后端（单进程，默认，使用 broadcaster）
                - redis://host:port/db - Redis Pub/Sub（默认使用原生后端）
                - redis-cluster://[password@]host:port - Redis Cluster
                - kafka://host:port - Apache Kafka（使用 broadcaster）
                - postgres://user:pass@host/db - PostgreSQL（使用 broadcaster）

        Returns:
            self: 支持链式调用
        """
        if self._initialized:
            logger.warning(f"通道管理器 [{self.name}] 已初始化，跳过")
            return self

        # 根据 URL scheme 自动选择后端
        if isinstance(backend, str):
            backend = ChannelBackend(backend.lower())

        self._backend_type = backend
        self._url = url

        # 根据后端类型选择实现
        if backend == ChannelBackend.REDIS:
            # 原生 Redis 后端（推荐，解决 broadcaster 并发问题）
            if url.startswith("memory://"):
                raise ValueError(
                    f"backend=redis 不支持 memory://，"
                    f"请使用 backend=broadcaster 或改用 redis:// URL"
                )
            redis_url = url
            if url.startswith("redis-cluster://"):
                redis_url = url.replace("redis-cluster://", "redis://")
                # 处理 password@host 格式（转换为标准 :password@host 格式）
                parsed = urlparse(redis_url)
                if parsed.username and not parsed.password:
                    redis_url = redis_url.replace(
                        f"redis://{parsed.username}@",
                        f"redis://:{parsed.username}@"
                    )
            self._backend = RedisChannel(redis_url)
        elif backend == ChannelBackend.BROADCASTER:
            # Broadcaster 后端（支持 memory/redis/kafka/postgres 等）
            broadcast_url = url
            if url.startswith("redis-cluster://"):
                broadcast_url = url.replace("redis-cluster://", "redis://")
                parsed = urlparse(broadcast_url)
                if parsed.username and not parsed.password:
                    broadcast_url = broadcast_url.replace(
                        f"redis://{parsed.username}@",
                        f"redis://:{parsed.username}@"
                    )
            self._backend = BroadcasterChannel(broadcast_url)
        elif backend in (ChannelBackend.RABBITMQ, ChannelBackend.ROCKETMQ):
            raise NotImplementedError(f"{backend.value} 后端暂未实现")
        else:
            raise ValueError(f"不支持的通道后端: {backend}")

        self._initialized = True
        logger.info(f"通道管理器 [{self.name}] 初始化完成: {backend.value}, url={self._mask_url(url)}")
        return self

    def _mask_url(self, url: str) -> str:
        """URL 脱敏（隐藏密码）。"""
        if "@" in url:
            parts = url.split("@")
            prefix = parts[0]
            suffix = parts[1]
            if ":" in prefix:
                scheme_and_user = prefix.rsplit(":", 1)[0]
                return f"{scheme_and_user}:***@{suffix}"
        return url

    @property
    def backend(self) -> IChannel:
        """获取通道后端。"""
        if self._backend is None:
            raise RuntimeError(
                f"通道管理器 [{self.name}] 未初始化，请先调用 initialize()"
            )
        return self._backend

    @property
    def backend_type(self) -> str:
        """获取当前后端类型。"""
        return self._backend_type.value if self._backend_type else "unknown"

    @property
    def is_initialized(self) -> bool:
        """检查是否已初始化。"""
        return self._initialized

    async def publish(
        self,
        channel: str,
        message: ChannelMessage | str | dict,
        *,
        event: str | None = None,
    ) -> None:
        """发布消息到通道。

        Args:
            channel: 通道名称
            message: 消息内容（ChannelMessage、字符串或字典）
            event: 事件类型（当 message 不是 ChannelMessage 时使用）
        """
        if isinstance(message, ChannelMessage):
            msg = message
        elif isinstance(message, dict):
            msg = ChannelMessage(data=message, event=event)
        else:
            msg = ChannelMessage(data=str(message), event=event)

        await self.backend.publish(channel, msg)

    async def subscribe(self, channel: str) -> AsyncIterator[ChannelMessage]:
        """订阅通道。

        Args:
            channel: 通道名称

        Yields:
            ChannelMessage: 接收到的消息
        """
        async for message in self.backend.subscribe(channel):
            yield message

    async def psubscribe(self, pattern: str) -> AsyncIterator[ChannelMessage]:
        """模式订阅（通配符）。

        Args:
            pattern: 通道模式，支持 * 和 ? 通配符
                - "space:123:*" 订阅 space:123 下所有事件
                - "user:*:notification" 订阅所有用户的通知

        Yields:
            ChannelMessage: 接收到的消息
        
        示例:
            # SSE 路由中订阅某个空间的所有事件
            async for msg in channel.psubscribe(f"space:{{space_id}}:*"):
                yield msg.to_sse()
            
            # 发布不同类型的事件
            await channel.publish(f"space:{{space_id}}:file_analyzed", {{...}})
            await channel.publish(f"space:{{space_id}}:comment_added", {{...}})
        """
        async for message in self.backend.psubscribe(pattern):
            yield message

    async def unsubscribe(self, channel: str) -> None:
        """取消订阅通道。

        Args:
            channel: 通道名称
        """
        await self.backend.unsubscribe(channel)

    async def cleanup(self) -> None:
        """清理资源，关闭通道。"""
        if self._backend:
            await self._backend.close()
            self._backend = None
        self._initialized = False
        logger.info(f"通道管理器 [{self.name}] 已关闭")

    def __repr__(self) -> str:
        """字符串表示。"""
        status = "initialized" if self._initialized else "not initialized"
        return f"<ChannelManager name={self.name} backend={self.backend_type} status={status}>"


__all__ = ["ChannelManager"]



"""共享配置基类。

提供所有应用共享的基础配置结构。
使用 pydantic-settings 进行分层分级配置管理。

注意：Application 层的配置是独立的，不依赖 Infrastructure 层。
"""

from __future__ import annotations

from pathlib import Path
from typing import Any, Literal

from dotenv import load_dotenv
from pydantic import BaseModel, Field
from pydantic_settings import BaseSettings, SettingsConfigDict

from .multi_instance import (
    MultiInstanceConfigLoader,
    MultiInstanceSettings,
    parse_multi_instance_env,
)


def _load_env_file(env_file: str | Path) -> bool:
    """加载 .env 文件到环境变量。"""
    return load_dotenv(env_file, override=True)


# =============================================================================
# 多实例配置基类
# =============================================================================


class DatabaseInstanceConfig(MultiInstanceSettings):
    """数据库实例配置。
    
    环境变量格式: DATABASE__{INSTANCE}__{FIELD}
    示例:
        DATABASE__DEFAULT__URL=postgresql://main...
        DATABASE__DEFAULT__POOL_SIZE=10
        DATABASE__ANALYTICS__URL=postgresql://analytics...
    """
    
    url: str = Field(
        default="sqlite+aiosqlite:///./app.db",
        description="数据库连接字符串"
    )
    echo: bool = Field(
        default=False,
        description="是否输出 SQL 语句"
    )
    pool_size: int = Field(
        default=5,
        description="数据库连接池大小"
    )
    max_overflow: int = Field(
        default=10,
        description="连接池最大溢出连接数"
    )
    pool_recycle: int = Field(
        default=3600,
        description="连接回收时间（秒）"
    )
    pool_timeout: int = Field(
        default=30,
        description="获取连接超时时间（秒）"
    )
    pool_pre_ping: bool = Field(
        default=True,
        description="是否在获取连接前进行 PING"
    )
    # 连接追踪配置（对标 HikariCP leakDetectionThreshold）
    tracker_enabled: bool = Field(
        default=False,
        description="是否启用连接追踪（生产环境建议关闭）"
    )
    leak_detection_threshold: int = Field(
        default=300,
        description="泄漏检测阈值（秒），连接持有超过此时间记录警告"
    )


class CacheInstanceConfig(MultiInstanceSettings):
    """缓存实例配置。
    
    环境变量格式: CACHE__{INSTANCE}__{FIELD}
    示例:
        CACHE__DEFAULT__BACKEND=redis
        CACHE__DEFAULT__URL=redis://localhost:6379/0
        CACHE__LOCAL__BACKEND=memory
        CACHE__LOCAL__MAX_SIZE=5000
    """
    
    backend: str = Field(
        default="memory",
        description="缓存后端 (memory/redis/memcached)"
    )
    url: str | None = Field(
        default=None,
        description="缓存服务 URL"
    )
    max_size: int = Field(
        default=1000,
        description="内存缓存最大大小"
    )


class StorageInstanceConfig(MultiInstanceSettings):
    """对象存储实例配置。
    
    环境变量格式: STORAGE__{INSTANCE}__{FIELD}
    示例:
        STORAGE__DEFAULT__BACKEND=s3
        STORAGE__DEFAULT__BUCKET=main-bucket
        STORAGE__BACKUP__BACKEND=local
        STORAGE__BACKUP__BASE_PATH=/backup
    """
    
    backend: Literal["local", "s3", "oss", "cos"] = Field(
        default="local",
        description="存储后端"
    )
    # S3 配置
    access_key_id: str | None = Field(default=None)
    access_key_secret: str | None = Field(default=None)
    endpoint: str | None = Field(default=None)
    region: str | None = Field(default=None)
    bucket_name: str | None = Field(default=None)
    # 本地存储
    base_path: str = Field(default="./storage")


class ChannelInstanceConfig(MultiInstanceSettings):
    """通道实例配置。
    
    环境变量格式: CHANNEL__{INSTANCE}__{FIELD}
    示例:
        CHANNEL__DEFAULT__BACKEND=memory
        CHANNEL__SHARED__BACKEND=redis
        CHANNEL__SHARED__URL=redis://localhost:6379/3
    """
    
    backend: str = Field(
        default="memory",
        description="通道后端 (memory/redis)"
    )
    url: str | None = Field(
        default=None,
        description="Redis URL（当 backend=redis 时需要）"
    )


class MQInstanceConfig(MultiInstanceSettings):
    """消息队列实例配置。
    
    环境变量格式: MQ__{INSTANCE}__{FIELD}
    示例:
        MQ__DEFAULT__BACKEND=redis_stream
        MQ__DEFAULT__URL=redis://localhost:6379/4
        MQ__DEFAULT__MAX_CONNECTIONS=1000
    """
    
    backend: str = Field(
        default="redis_stream",
        description="消息队列后端 (redis/redis_stream/rabbitmq)"
    )
    url: str | None = Field(
        default=None,
        description="连接 URL"
    )
    max_connections: int = Field(
        default=1000,
        description="Redis 连接池最大连接数"
    )


class EventInstanceConfig(MultiInstanceSettings):
    """事件总线实例配置。
    
    环境变量格式: EVENT__{INSTANCE}__{FIELD}
    示例:
        EVENT__DEFAULT__BACKEND=memory
        EVENT__DISTRIBUTED__BACKEND=redis
        EVENT__DISTRIBUTED__URL=redis://localhost:6379/5
    """
    
    backend: str = Field(
        default="memory",
        description="事件后端 (memory/redis/rabbitmq)"
    )
    url: str | None = Field(
        default=None,
        description="连接 URL"
    )


# =============================================================================
# 单实例配置
# =============================================================================


class DatabaseSettings(BaseModel):
    """数据库配置（单实例）。
    
    环境变量格式: DATABASE__{FIELD}
    示例: DATABASE__URL, DATABASE__POOL_SIZE
    多实例格式: DATABASE__{INSTANCE}__{FIELD}
    """
    
    url: str = Field(
        default="sqlite+aiosqlite:///./app.db",
        description="数据库连接字符串"
    )
    echo: bool = Field(
        default=False,
        description="是否输出 SQL 语句"
    )
    pool_size: int = Field(
        default=5,
        description="数据库连接池大小"
    )
    max_overflow: int = Field(
        default=10,
        description="连接池最大溢出连接数"
    )
    pool_recycle: int = Field(
        default=3600,
        description="连接回收时间（秒）"
    )
    pool_timeout: int = Field(
        default=30,
        description="获取连接超时时间（秒）"
    )
    pool_pre_ping: bool = Field(
        default=True,
        description="是否在获取连接前进行 PING"
    )
    slow_query_threshold: float = Field(
        default=1.0,
        description="慢查询阈值（秒），超过此时间的查询会记录警告日志"
    )
    # 连接追踪配置（对标 HikariCP leakDetectionThreshold）
    tracker_enabled: bool = Field(
        default=False,
        description="是否启用连接追踪（生产环境建议关闭）"
    )
    leak_detection_threshold: int = Field(
        default=300,
        description="泄漏检测阈值（秒），连接持有超过此时间记录警告"
    )


class CacheSettings(BaseModel):
    """缓存配置。
    
    环境变量格式: CACHE__{FIELD}
    示例: CACHE__TYPE, CACHE__URL, CACHE__MAX_SIZE
    
    支持的缓存类型：
    - memory: 内存缓存（默认，无需 URL）
    - redis: Redis 缓存（需要设置 CACHE__URL）
    - memcached: Memcached 缓存（需要设置 CACHE__URL）
    """
    
    cache_type: str = Field(
        default="memory",
        description="缓存类型 (memory/redis/memcached)"
    )
    url: str | None = Field(
        default=None,
        description="缓存服务 URL（如 redis://localhost:6379）"
    )
    max_size: int = Field(
        default=1000,
        description="内存缓存最大大小"
    )


class ChannelSettings(BaseModel):
    """流式通道配置（单实例）。
    
    环境变量格式: CHANNEL__{FIELD}
    示例: CHANNEL__BACKEND, CHANNEL__URL
    
    支持的后端类型：
    - memory: 内存后端（默认，单进程）
    - redis: Redis Pub/Sub（多进程/分布式）
    - redis_cluster: Redis Cluster Sharded Pub/Sub（Redis 7.0+）
    
    注意：URL 的 scheme 会自动决定后端类型：
    - redis-cluster://... 自动使用 redis_cluster 后端
    """
    
    backend: str = Field(
        default="",
        description="通道后端 (memory/redis/redis_cluster)，空字符串表示不启用"
    )
    url: str | None = Field(
        default=None,
        description="连接 URL（redis://... 或 redis-cluster://...）"
    )


class StorageSettings(BaseModel):
    """对象存储组件接入配置（Application 层）。

    环境变量格式: STORAGE__{FIELD}
    示例: STORAGE__TYPE, STORAGE__BUCKET_NAME
    """

    enabled: bool = Field(default=True, description="是否启用存储组件")

    # 后端类型
    type: Literal["local", "s3", "oss", "cos"] = Field(default="local", description="存储类型")

    # S3/兼容协议通用
    access_key_id: str | None = Field(default=None, description="访问密钥ID")
    access_key_secret: str | None = Field(default=None, description="访问密钥")
    session_token: str | None = Field(default=None, description="会话令牌（STS临时凭证）")
    endpoint: str | None = Field(default=None, description="端点URL（MinIO/私有云等）")
    region: str | None = Field(default=None, description="区域")
    bucket_name: str | None = Field(default=None, description="默认桶名")
    addressing_style: Literal["virtual", "path"] = Field(default="virtual", description="S3寻址风格")

    # S3 AssumeRole（服务端使用；由外部决定何时刷新）
    role_arn: str | None = Field(default=None, description="STS AssumeRole 的角色ARN")
    role_session_name: str = Field(default="aury-storage", description="STS会话名")
    external_id: str | None = Field(default=None, description="STS ExternalId")
    sts_endpoint: str | None = Field(default=None, description="STS端点（可选）")
    sts_region: str | None = Field(default=None, description="STS区域（可选）")
    sts_duration_seconds: int = Field(default=3600, description="AssumeRole DurationSeconds")

    # local
    base_path: str = Field(default="./storage", description="本地存储基础目录")


class ServerSettings(BaseModel):
    """服务器配置。
    
    环境变量格式: SERVER__{FIELD}
    示例: SERVER__HOST, SERVER__PORT, SERVER__RELOAD
    """
    
    host: str = Field(
        default="0.0.0.0",
        description="服务器监听地址"
    )
    port: int = Field(
        default=8000,
        description="服务器监听端口"
    )
    reload: bool = Field(
        default=True,
        description="是否启用热重载"
    )
    workers: int = Field(
        default=1,
        description="工作进程数"
    )


class CORSSettings(BaseModel):
    """CORS配置。
    
    环境变量格式: CORS__{FIELD}
    示例: CORS__ORIGINS, CORS__ALLOW_CREDENTIALS, CORS__ALLOW_METHODS
    """
    
    origins: list[str] = Field(
        default=["*"],
        description="允许的CORS源"
    )
    allow_credentials: bool = Field(
        default=True,
        description="是否允许CORS凭据"
    )
    allow_methods: list[str] = Field(
        default=["*"],
        description="允许的CORS方法"
    )
    allow_headers: list[str] = Field(
        default=["*"],
        description="允许的CORS头"
    )


class LogSettings(BaseModel):
    """日志配置。
    
    环境变量格式: LOG__{FIELD}
    示例: LOG__LEVEL, LOG__DIR, LOG__ROTATION_TIME, LOG__RETENTION_DAYS
    """
    
    level: str = Field(
        default="INFO",
        description="日志级别 (DEBUG/INFO/WARNING/ERROR/CRITICAL)"
    )
    dir: str | None = Field(
        default=None,
        description="日志文件目录（如果不设置则默认为 './log'）"
    )
    rotation_time: str = Field(
        default="00:00",
        description="日志文件轮转时间 (HH:MM 格式，每天定时轮转)"
    )
    rotation_size: str = Field(
        default="50 MB",
        description="日志文件轮转大小阈值（超过此大小会产生 .1, .2 等后缀文件）"
    )
    retention_days: int = Field(
        default=7,
        description="日志文件保留天数"
    )
    enable_file_rotation: bool = Field(
        default=True,
        description="是否启用日志文件轮转"
    )
    enable_classify: bool = Field(
        default=True,
        description="是否按模块和级别分类日志文件"
    )
    enable_console: bool = Field(
        default=True,
        description="是否输出日志到控制台"
    )
    websocket_log_messages: bool = Field(
        default=False,
        description="是否记录 WebSocket 消息内容（注意性能和敏感数据）"
    )
    enqueue: bool = Field(
        default=True,
        description=(
            "是否启用后台线程写入日志。"
            "启用后日志在单独线程写入文件，避免阻塞事件循环。"
            "asyncio 应用必须设为 True，否则文件 I/O 会阻塞事件循环"
        )
    )


class ServiceSettings(BaseModel):
    """服务配置。

    环境变量格式: SERVICE__{FIELD}
    示例: SERVICE__NAME, SERVICE__TYPE, SERVICE__ENVIRONMENT

    服务类型说明：
    - api: 运行 API 服务（SCHEDULER__ENABLED 决定是否同时运行调度器）
    - worker: 运行任务队列 Worker（处理异步任务）

    独立调度器通过 `aury scheduler` 命令运行，不需要配置 SERVICE__TYPE。
    """

    name: str = Field(
        default="app",
        description="服务名称，用于日志目录区分、链路追踪标识"
    )
    version: str = Field(
        default="",
        description="服务版本（用于链路追踪和监控）"
    )
    environment: str = Field(
        default="development",
        description="部署环境 (development/staging/production)"
    )
    service_type: str = Field(
        default="api",
        description="服务类型（api/worker）"
    )


class SchedulerSettings(BaseModel):
    """调度器配置。
    
    环境变量格式: SCHEDULER__{FIELD}
    示例: SCHEDULER__ENABLED, SCHEDULER__SCHEDULE_MODULES
    
    仅在 SERVICE__TYPE=api 时有效：
    - SCHEDULER__ENABLED=true: API 服务同时运行内嵌调度器（默认）
    - SCHEDULER__ENABLED=false: 只运行 API，不启动调度器
    
    独立调度器通过 `aury scheduler` 命令运行，不需要此配置。
    
    分布式调度：
    - 配置 SCHEDULER__JOBSTORE_URL 使用 Redis/SQLAlchemy 存储
    - 多节点部署时共享任务状态
    """
    
    enabled: bool = Field(
        default=True,
        description="是否在 API 服务中启用内嵌调度器"
    )
    schedule_modules: list[str] = Field(
        default_factory=list,
        description="定时任务模块列表。为空时自动发现 schedules 模块"
    )
    # APScheduler 配置
    jobstore_url: str | None = Field(
        default=None,
        description=(
            "任务存储 URL。支持：\n"
            "- redis-cluster://password@host:port（Redis Cluster 存储）\n"
            "- redis://localhost:6379/0（Redis 存储）\n"
            "- sqlite:///jobs.db（SQLite 存储）\n"
            "- postgresql://user:pass@host/db（PostgreSQL 存储）\n"
            "- 不配置则使用内存存储"
        )
    )
    timezone: str | None = Field(
        default=None,
        description="调度器时区，如 Asia/Shanghai、UTC"
    )
    coalesce: bool = Field(
        default=True,
        description="是否合并错过的任务执行（多次错过只执行一次）"
    )
    max_instances: int = Field(
        default=1,
        description="同一任务的最大并发实例数"
    )
    misfire_grace_time: int = Field(
        default=60,
        description="任务错过容忍时间（秒），超过此时间则跳过"
    )


class TaskSettings(BaseModel):
    """任务队列配置。
    
    环境变量格式: TASK__{FIELD}
    示例: TASK__BROKER_URL, TASK__MAX_RETRIES
    """
    
    broker_url: str | None = Field(
        default=None,
        description="任务队列代理 URL（如 Redis 或 RabbitMQ）"
    )
    max_retries: int = Field(
        default=3,
        description="最大重试次数"
    )
    timeout: int = Field(
        default=3600,
        description="任务超时时间（秒）"
    )


class EventSettings(BaseModel):
    """事件总线配置。
    
    环境变量格式: EVENT__{FIELD}
    示例: EVENT__BROKER_URL, EVENT__EXCHANGE_NAME
    """
    
    broker_url: str | None = Field(
        default=None,
        description="事件总线代理 URL（如 RabbitMQ）"
    )
    exchange_name: str = Field(
        default="aury.events",
        description="事件交换机名称"
    )


class MessageQueueSettings(BaseModel):
    """消息队列配置。
    
    环境变量格式: MQ__{FIELD}
    示例: MQ__BACKEND, MQ__URL, MQ__BROKER_URL
    
    与 Task（任务队列）的区别：
    - Task: 基于 Dramatiq，用于异步任务处理（API + Worker 模式）
    - MQ: 通用消息队列，用于服务间通信、事件驱动架构
    
    支持的后端：
    - redis: redis://localhost:6379/0
    - redis_stream: redis://localhost:6379/0
    - rabbitmq: amqp://guest:guest@localhost:5672//
    """
    
    backend: str = Field(
        default="",
        description="消息队列后端 (redis/redis_stream/rabbitmq)，空字符串表示不启用"
    )
    url: str | None = Field(
        default=None,
        description="连接 URL"
    )
    default_queue: str = Field(
        default="default",
        description="默认队列名称"
    )
    serializer: str = Field(
        default="json",
        description="序列化方式（json/pickle/msgpack）"
    )
    prefetch_count: int = Field(
        default=1,
        description="预取消息数量"
    )


class TelemetrySettings(BaseModel):
    """OpenTelemetry 配置。
    
    环境变量格式: TELEMETRY__{FIELD}
    示例: TELEMETRY__ENABLED, TELEMETRY__SLOW_THRESHOLD
    
    功能说明：
    - 启用后自动 instrument FastAPI、SQLAlchemy、httpx
    - get_trace_id() 会优先使用 OTel trace_id
    - 可配置 AlertingSpanProcessor 自动检测慢请求/异常并触发告警
    - 可选配置 OTLP 导出到 Jaeger/Tempo/Collector
    
    注意：service_name/version/environment 从 ServiceSettings 获取。
    """
    
    enabled: bool = Field(
        default=False,
        description="是否启用 OpenTelemetry"
    )
    
    # Instrumentation 开关
    instrument_fastapi: bool = Field(
        default=True,
        description="是否自动 instrument FastAPI"
    )
    instrument_sqlalchemy: bool = Field(
        default=True,
        description="是否自动 instrument SQLAlchemy"
    )
    instrument_httpx: bool = Field(
        default=True,
        description="是否自动 instrument httpx"
    )
    
    
    # OTLP Traces 导出配置
    traces_endpoint: str | None = Field(
        default=None,
        description="Traces 导出端点（如 http://jaeger:4317）"
    )
    traces_headers: dict[str, str] = Field(
        default_factory=dict,
        description="Traces 导出请求头"
    )
    
    # OTLP Logs 导出配置
    logs_endpoint: str | None = Field(
        default=None,
        description="Logs 导出端点（如 http://loki:3100）"
    )
    logs_headers: dict[str, str] = Field(
        default_factory=dict,
        description="Logs 导出请求头"
    )
    
    # OTLP Metrics 导出配置
    metrics_endpoint: str | None = Field(
        default=None,
        description="Metrics 导出端点（如 http://prometheus:9090）"
    )
    metrics_headers: dict[str, str] = Field(
        default_factory=dict,
        description="Metrics 导出请求头"
    )
    
    # 采样配置
    sampling_rate: float = Field(
        default=1.0,
        description="采样率 (0.0-1.0)，1.0 表示 100%"
    )


class AlertSettings(BaseModel):
    """告警系统配置。
    
    环境变量格式: ALERT__{FIELD}
    示例: ALERT__ENABLED, ALERT__RULES_FILE
    
    通知器配置（作为子配置，动态字段）：
        ALERT__NOTIFIERS__FEISHU__TYPE=feishu
        ALERT__NOTIFIERS__FEISHU__WEBHOOK=https://open.feishu.cn/...
        ALERT__NOTIFIERS__FEISHU__SECRET=xxx
        
        ALERT__NOTIFIERS__OPS__TYPE=webhook
        ALERT__NOTIFIERS__OPS__URL=https://my-system.com/alert
    
    自定义通知器通过 AlertManager.register_notifier() 注册。
    """
    
    # 通知器配置缓存（动态字段）
    _notifiers: dict[str, dict[str, Any]] | None = None
    
    enabled: bool = Field(
        default=False,
        description="是否启用告警系统"
    )
    rules_file: str | None = Field(
        default=None,
        description="告警规则文件路径（YAML 格式），如 alert_rules.yaml"
    )
    
    # 慢操作阈值
    slow_request_threshold: float = Field(
        default=1.0,
        description="慢请求阈值（秒）"
    )
    slow_sql_threshold: float = Field(
        default=0.5,
        description="慢 SQL 阈值（秒）"
    )
    
    # 告警开关
    alert_on_slow_request: bool = Field(
        default=True,
        description="是否对慢 HTTP 请求发送告警"
    )
    alert_on_slow_sql: bool = Field(
        default=True,
        description="是否对慢 SQL 发送告警"
    )
    alert_on_error: bool = Field(
        default=True,
        description="是否对异常发送告警（默认只对 5xx 告警，4xx 业务异常不告警）"
    )
    
    # 慢请求路径排除配置
    slow_request_exclude_paths: list[str] = Field(
        default_factory=list,
        description="排除慢请求告警的路径列表（支持 * 通配符），如 SSE/WebSocket 长连接接口"
    )
    
    # 默认累计触发配置
    aggregate_window: int = Field(
        default=10,
        description="聚合窗口（秒）"
    )
    slow_request_aggregate: int = Field(
        default=5,
        description="慢请求触发阈值（窗口内次数）"
    )
    slow_sql_aggregate: int = Field(
        default=10,
        description="慢 SQL 触发阈值（窗口内次数）"
    )
    exception_aggregate: int = Field(
        default=1,
        description="异常触发阈值（通常为 1，立即告警）"
    )
    
    # 抑制配置
    suppress_seconds: int = Field(
        default=300,
        description="告警抑制时间（秒），相同告警在此时间内不重复发送"
    )
    
    def get_notifiers(self) -> dict[str, dict[str, Any]]:
        """获取所有告警通知器实例配置。
        
        从环境变量解析 ALERT__NOTIFIERS__{INSTANCE}__{FIELD} 格式的配置。
        支持动态字段，不同类型通知器可有不同字段。
        
        Returns:
            dict[str, dict[str, Any]]: 实例名 -> 配置字典
            
        示例:
            ALERT__NOTIFIERS__FEISHU__TYPE=feishu
            ALERT__NOTIFIERS__FEISHU__WEBHOOK=https://...
            
            返回: {"feishu": {"type": "feishu", "webhook": "https://..."}}
        
        自定义通知器通过 AlertManager.register_notifier() 注册。
        """
        if self._notifiers is None:
            self._notifiers = parse_multi_instance_env("ALERT__NOTIFIERS")
        return self._notifiers


class PrometheusMetricsSettings(BaseModel):
    """Prometheus Metrics 配置。
    
    环境变量格式: PROMETHEUS_METRICS__{FIELD}
    示例: PROMETHEUS_METRICS__ENABLED, PROMETHEUS_METRICS__PATH
    
    功能说明：
    - 启用后自动暴露 /metrics 端点供 Prometheus 抓取
    - 统计 HTTP 请求数、延迟、活跃连接数等指标
    - 需要安装: prometheus-fastapi-instrumentator
    """
    
    enabled: bool = Field(
        default=False,
        description="是否启用 Prometheus Metrics"
    )
    path: str = Field(
        default="/api/metrics",
        description="Metrics 端点路径"
    )
    include_in_schema: bool = Field(
        default=False,
        description="是否在 OpenAPI schema 中显示 /metrics 端点"
    )
    # 指标配置
    should_group_status_codes: bool = Field(
        default=True,
        description="是否按状态码分组 (2xx, 3xx, 4xx, 5xx)"
    )
    should_ignore_untemplated: bool = Field(
        default=False,
        description="是否忽略没有模板路由的请求"
    )
    should_group_untemplated: bool = Field(
        default=True,
        description="是否将没有模板路由的请求归为一组"
    )
    should_round_latency_decimals: bool = Field(
        default=False,
        description="是否四舍五入延迟小数"
    )
    should_respect_env_var: bool = Field(
        default=False,
        description="是否使用 ENABLE_METRICS 环境变量控制"
    )
    should_instrument_requests_inprogress: bool = Field(
        default=True,
        description="是否统计进行中的请求数（活跃连接）"
    )
    excluded_handlers: list[str] = Field(
        default_factory=lambda: ["/api/metrics", "/health", "/api/health"],
        description="排除的路径列表（不统计这些路径的指标）"
    )


class ProfilingSettings(BaseModel):
    """Profiling 配置。
    
    环境变量格式: PROFILING__{FIELD}
    示例: PROFILING__ENABLED, PROFILING__PYROSCOPE_ENDPOINT
    
    功能说明：
    - Pyroscope：持续采样生成火焰图（需安装 pyroscope-io）
    - 阻塞检测：检测同步代码阻塞事件循环（需安装 psutil）
    """
    
    # Pyroscope 持续 Profiling
    enabled: bool = Field(
        default=False,
        description="是否启用 Pyroscope 持续 profiling"
    )
    pyroscope_endpoint: str | None = Field(
        default=None,
        description="Pyroscope 服务端点（如 http://pyroscope:4040）"
    )
    pyroscope_auth_token: str | None = Field(
        default=None,
        description="Pyroscope 认证 token（可选）"
    )
    pyroscope_sample_rate: int = Field(
        default=100,
        description="Pyroscope 采样率 (Hz)，降低可减少开销"
    )
    pyroscope_tags: dict[str, str] = Field(
        default_factory=dict,
        description="Pyroscope 自定义标签"
    )
    
    # 事件循环阻塞检测
    blocking_detector_enabled: bool = Field(
        default=False,
        description="是否启用事件循环阻塞检测"
    )
    blocking_check_interval_ms: float = Field(
        default=100,
        description="阻塞检测间隔 (ms)"
    )
    blocking_threshold_ms: float = Field(
        default=100,
        description="阻塞阈值 (ms)，超过此时间记录阻塞事件"
    )
    blocking_severe_threshold_ms: float = Field(
        default=500,
        description="严重阻塞阈值 (ms)，超过此时间触发严重告警"
    )
    blocking_alert_enabled: bool = Field(
        default=True,
        description="检测到阻塞时是否发送告警"
    )
    blocking_alert_cooldown_seconds: float = Field(
        default=60,
        description="阻塞告警冷却时间 (秒)，避免告警风暴"
    )
    blocking_max_history: int = Field(
        default=50,
        description="保留的阻塞事件历史数量"
    )
    
    # GC 调优（高并发流式应用建议启用）
    gc_threshold_gen0: int = Field(
        default=700,
        description="GC gen0 阈值（默认 700）。流式应用建议 2000-3000 减少 GC 频率"
    )
    gc_threshold_gen1: int = Field(
        default=10,
        description="GC gen1 阈值（默认 10）"
    )
    gc_threshold_gen2: int = Field(
        default=10,
        description="GC gen2 阈值（默认 10）"
    )


class MigrationSettings(BaseModel):
    """数据库迁移配置。
    
    环境变量格式: MIGRATION__{FIELD}
    示例: MIGRATION__CONFIG_PATH, MIGRATION__SCRIPT_LOCATION, MIGRATION__MODEL_MODULES
    """
    
    config_path: str = Field(
        default="alembic.ini",
        description="Alembic 配置文件路径"
    )
    script_location: str = Field(
        default="migrations",
        description="Alembic 迁移脚本目录"
    )
    model_modules: list[str] = Field(
        default_factory=lambda: [
            "models",
            "app.models",
            "app.**.models",
        ],
        description="模型模块列表（用于自动检测变更）。支持通配符: * 和 **。"
    )
    auto_create: bool = Field(
        default=True,
        description="是否自动创建迁移配置和目录"
    )


class RPCClientSettings(BaseModel):
    """RPC 客户端调用配置。
    
    用于配置客户端调用其他服务时的行为。
    
    环境变量格式: RPC_CLIENT__{FIELD}
    示例: RPC_CLIENT__SERVICES, RPC_CLIENT__TIMEOUT, RPC_CLIENT__RETRY_TIMES
    """
    
    services: dict[str, str] = Field(
        default_factory=dict,
        description="服务地址映射 {service_name: url}（优先级最高，会覆盖 DNS 解析）"
    )
    default_timeout: int = Field(
        default=30,
        description="默认超时时间（秒）"
    )
    default_retry_times: int = Field(
        default=3,
        description="默认重试次数"
    )
    dns_scheme: str = Field(
        default="http",
        description="DNS 解析使用的协议（http 或 https）"
    )
    dns_port: int = Field(
        default=80,
        description="DNS 解析默认端口"
    )
    use_dns_fallback: bool = Field(
        default=True,
        description="是否在配置中找不到时使用 DNS 解析（K8s/Docker Compose 自动 DNS）"
    )


class RPCServiceSettings(BaseModel):
    """RPC 服务注册配置。
    
    用于配置当前服务注册到服务注册中心时的信息。
    
    环境变量格式: RPC_SERVICE__{FIELD}
    示例: RPC_SERVICE__NAME, RPC_SERVICE__URL, RPC_SERVICE__HEALTH_CHECK_URL
    """
    
    name: str | None = Field(
        default=None,
        description="服务名称（用于注册）"
    )
    url: str | None = Field(
        default=None,
        description="服务地址（用于注册）"
    )
    health_check_url: str | None = Field(
        default=None,
        description="健康检查 URL（用于注册）"
    )
    auto_register: bool = Field(
        default=False,
        description="是否自动注册到服务注册中心"
    )
    registry_url: str | None = Field(
        default=None,
        description="服务注册中心地址（如果使用外部注册中心）"
    )


class HealthCheckSettings(BaseModel):
    """健康检查配置。
    
    用于配置 Aury 框架的默认健康检查端点。
    注意：此配置仅用于框架内置的健康检查端点，不影响服务自身的健康检查端点。
    
    环境变量格式: HEALTH_CHECK__{FIELD}
    示例: HEALTH_CHECK__PATH, HEALTH_CHECK__ENABLED
    """
    
    path: str = Field(
        default="/api/health",
        description="健康检查端点路径（默认: /api/health）"
    )
    enabled: bool = Field(
        default=True,
        description="是否启用 Aury 默认健康检查端点"
    )


class AdminAuthSettings(BaseModel):
    """管理后台认证配置。

    作为 ADMIN 配置的子配置，环境变量格式: ADMIN__AUTH__{FIELD}
    示例: ADMIN__AUTH__MODE, ADMIN__AUTH__SECRET_KEY, ADMIN__AUTH__BASIC_USERNAME

    说明：
    - 内置模式仅保证 basic / bearer 开箱即用
    - jwt/custom 推荐由用户自定义 backend 实现
    """

    mode: Literal["none", "basic", "bearer", "jwt", "custom"] = Field(
        default="basic",
        description="认证模式 (none/basic/bearer/jwt/custom)",
    )

    # SQLAdmin AuthenticationBackend 需要 secret_key 用于 session 签名
    secret_key: str | None = Field(
        default=None,
        description="Session 签名密钥（生产环境必须配置）",
    )

    # basic：用户名/密码（用于 SQLAdmin 登录页）
    basic_username: str | None = Field(default=None, description="Basic 登录用户名")
    basic_password: str | None = Field(default=None, description="Basic 登录密码")

    # bearer：token 白名单（支持 Authorization: Bearer <token>，也支持登录页使用 token）
    bearer_tokens: list[str] = Field(default_factory=list, description="Bearer token 白名单")

    # custom/jwt：用户自定义认证后端（动态导入）
    backend: str | None = Field(
        default=None,
        description='自定义认证后端导入路径，如 "yourpkg.admin_auth:backend"',
    )


class AdminConsoleSettings(BaseModel):
    """SQLAdmin 管理后台配置。

    环境变量格式: ADMIN__{FIELD}
    示例: ADMIN__ENABLED, ADMIN__PATH, ADMIN__DATABASE_URL
    嵌套配置: ADMIN__AUTH__{FIELD}
    """

    enabled: bool = Field(default=False, description="是否启用管理后台")

    path: str = Field(
        default="/api/admin-console",
        description="管理后台路径（默认 /api/admin-console）",
    )

    # SQLAdmin 目前通常要求同步 SQLAlchemy Engine；此处允许单独指定同步数据库 URL
    database_url: str | None = Field(
        default=None,
        description="管理后台专用数据库连接（同步 URL，可覆盖自动推导）",
    )

    # 显式指定项目侧 admin 模块路径（可选）
    views_module: str | None = Field(
        default=None,
        description="项目侧 admin-console 模块（用于注册 views/auth），如 app.admin_console",
    )

    auth: AdminAuthSettings = Field(default_factory=AdminAuthSettings, description="管理后台认证配置")


class BaseConfig(BaseSettings):
    """基础配置类。
    
    所有应用配置的基类，提供通用配置项。
    初始化时自动从 .env 文件加载环境变量，然后由 pydantic-settings 读取环境变量。
    
    环境变量格式:
    使用双下划线 (__) 作为层级分隔符：
        {SECTION}__{FIELD}=value
        {SECTION}__{NESTED}__{FIELD}=value
    
    单实例示例:
        SERVER__HOST=0.0.0.0
        SERVER__PORT=8000
        DATABASE__URL=postgresql://...
        LOG__LEVEL=INFO
    
    多实例示例:
        DATABASE__DEFAULT__URL=postgresql://main...
        DATABASE__ANALYTICS__URL=postgresql://analytics...
        CACHE__DEFAULT__BACKEND=redis
        CACHE__REDIS__URL=redis://localhost:6379/1
    
    注意：Application 层配置完全独立，不依赖 Infrastructure 层。
    """
    
    # 多实例配置缓存
    _databases: dict[str, DatabaseInstanceConfig] | None = None
    _caches: dict[str, CacheInstanceConfig] | None = None
    _storages: dict[str, StorageInstanceConfig] | None = None
    _channels: dict[str, ChannelInstanceConfig] | None = None
    _mqs: dict[str, MQInstanceConfig] | None = None
    _events: dict[str, EventInstanceConfig] | None = None
    
    def __init__(self, _env_file: str | Path = ".env", **kwargs) -> None:
        """初始化配置。
        
        Args:
            _env_file: .env 文件路径，默认为当前目录下的 .env
            **kwargs: 其他配置参数
        """
        # 在 pydantic-settings 初始化之前加载 .env 文件
        _load_env_file(_env_file)
        super().__init__(**kwargs)
    
    # ========== 服务器与网络 ==========
    # 服务器配置
    server: ServerSettings = Field(default_factory=ServerSettings)
    
    # CORS配置
    cors: CORSSettings = Field(default_factory=CORSSettings)
    
    # 日志配置
    log: LogSettings = Field(default_factory=LogSettings)
    
    # 健康检查配置
    health_check: HealthCheckSettings = Field(default_factory=HealthCheckSettings)

    # 管理后台配置（SQLAdmin）
    admin: AdminConsoleSettings = Field(default_factory=AdminConsoleSettings)
    
    # ========== 数据与缓存 ==========
    database: DatabaseSettings = Field(default_factory=DatabaseSettings)
    cache: CacheSettings = Field(default_factory=CacheSettings)
    channel: ChannelSettings = Field(default_factory=ChannelSettings)
    storage: StorageSettings = Field(default_factory=StorageSettings)
    
    # 迁移配置
    migration: MigrationSettings = Field(default_factory=MigrationSettings)
    
    # ========== 服务编排 ==========
    # 服务配置
    service: ServiceSettings = Field(default_factory=ServiceSettings)
    
    # 调度器配置
    scheduler: SchedulerSettings = Field(default_factory=SchedulerSettings)
    
    # ========== 异步与事件 ==========
    task: TaskSettings = Field(default_factory=TaskSettings)
    event: EventSettings = Field(default_factory=EventSettings)
    mq: MessageQueueSettings = Field(default_factory=MessageQueueSettings)
    
    # ========== 微服务通信 ==========
    # RPC 客户端配置（调用其他服务）
    rpc_client: RPCClientSettings = Field(default_factory=RPCClientSettings)
    
    # RPC 服务配置（当前服务注册）
    rpc_service: RPCServiceSettings = Field(default_factory=RPCServiceSettings)
    
    # ========== 监控告警 ==========
    telemetry: TelemetrySettings = Field(default_factory=TelemetrySettings)
    alert: AlertSettings = Field(default_factory=AlertSettings)
    profiling: ProfilingSettings = Field(default_factory=ProfilingSettings)
    prometheus_metrics: PrometheusMetricsSettings = Field(default_factory=PrometheusMetricsSettings)
    
    model_config = SettingsConfigDict(
        case_sensitive=False,
        extra="ignore",
        env_nested_delimiter="__",
    )
    
    # ========== 多实例配置访问方法 ==========
    
    def get_databases(self) -> dict[str, DatabaseInstanceConfig]:
        """获取所有数据库实例配置。
        
        从环境变量解析 DATABASE__{INSTANCE}__{FIELD} 格式的配置。
        如果没有配置多实例，返回从单实例配置转换的 default 实例。
        """
        if self._databases is None:
            loader = MultiInstanceConfigLoader("DATABASE", DatabaseInstanceConfig)
            self._databases = loader.load()
            if not self._databases:
                self._databases = {
                    "default": DatabaseInstanceConfig(
                        url=self.database.url,
                        echo=self.database.echo,
                        pool_size=self.database.pool_size,
                        max_overflow=self.database.max_overflow,
                        pool_recycle=self.database.pool_recycle,
                        pool_timeout=self.database.pool_timeout,
                        pool_pre_ping=self.database.pool_pre_ping,
                    )
                }
        return self._databases
    
    def get_caches(self) -> dict[str, CacheInstanceConfig]:
        """获取所有缓存实例配置。
        
        从环境变量解析 CACHE__{INSTANCE}__{FIELD} 格式的配置。
        如果没有配置多实例，返回从单实例配置转换的 default 实例。
        """
        if self._caches is None:
            loader = MultiInstanceConfigLoader("CACHE", CacheInstanceConfig)
            self._caches = loader.load()
            if not self._caches:
                self._caches = {
                    "default": CacheInstanceConfig(
                        backend=self.cache.cache_type,
                        url=self.cache.url,
                        max_size=self.cache.max_size,
                    )
                }
        return self._caches
    
    def get_storages(self) -> dict[str, StorageInstanceConfig]:
        """获取所有存储实例配置。
        
        从环境变量解析 STORAGE__{INSTANCE}__{FIELD} 格式的配置。
        如果没有配置多实例，返回从单实例配置转换的 default 实例。
        """
        if self._storages is None:
            loader = MultiInstanceConfigLoader("STORAGE", StorageInstanceConfig)
            self._storages = loader.load()
            if not self._storages:
                self._storages = {
                    "default": StorageInstanceConfig(
                        backend=self.storage.type,
                        access_key_id=self.storage.access_key_id,
                        access_key_secret=self.storage.access_key_secret,
                        endpoint=self.storage.endpoint,
                        region=self.storage.region,
                        bucket_name=self.storage.bucket_name,
                        base_path=self.storage.base_path,
                    )
                }
        return self._storages
    
    def get_channels(self) -> dict[str, ChannelInstanceConfig]:
        """获取所有通道实例配置。
        
        从环境变量解析 CHANNEL__{INSTANCE}__{FIELD} 格式的配置。
        如果没有配置多实例，返回从单实例配置转换的 default 实例。
        """
        if self._channels is None:
            loader = MultiInstanceConfigLoader("CHANNEL", ChannelInstanceConfig)
            self._channels = loader.load()
            if not self._channels and self.channel.backend:
                self._channels = {
                    "default": ChannelInstanceConfig(
                        backend=self.channel.backend,
                        url=self.channel.url,
                    )
                }
        return self._channels
    
    def get_mqs(self) -> dict[str, MQInstanceConfig]:
        """获取所有消息队列实例配置。
        
        从环境变量解析 MQ__{INSTANCE}__{FIELD} 格式的配置。
        如果没有配置多实例，返回从单实例配置转换的 default 实例。
        """
        if self._mqs is None:
            loader = MultiInstanceConfigLoader("MQ", MQInstanceConfig)
            self._mqs = loader.load()
            if not self._mqs and self.mq.backend:
                self._mqs = {
                    "default": MQInstanceConfig(
                        backend=self.mq.backend,
                        url=self.mq.url,
                    )
                }
        return self._mqs
    
    def get_events(self) -> dict[str, EventInstanceConfig]:
        """获取所有事件总线实例配置。
        
        从环境变量解析 EVENT__{INSTANCE}__{FIELD} 格式的配置。
        如果没有配置多实例，返回从单实例配置转换的 default 实例。
        """
        if self._events is None:
            loader = MultiInstanceConfigLoader("EVENT", EventInstanceConfig)
            self._events = loader.load()
            if not self._events and self.event.broker_url:
                self._events = {
                    "default": EventInstanceConfig(
                        backend="redis" if "redis" in (self.event.broker_url or "") else "rabbitmq",
                        url=self.event.broker_url,
                    )
                }
        return self._events
    
    @property
    def is_production(self) -> bool:
        """是否为生产环境。"""
        return self.log.level.upper() == "ERROR"


__all__ = [
    # 配置类
    "AdminAuthSettings",
    "AdminConsoleSettings",
    "AlertSettings",
    "BaseConfig",
    "CORSSettings",
    # 多实例配置类
    "CacheInstanceConfig",
    "CacheSettings",
    "ChannelInstanceConfig",
    "ChannelSettings",
    "DatabaseInstanceConfig",
    "DatabaseSettings",
    "EventInstanceConfig",
    "EventSettings",
    "HealthCheckSettings",
    "LogSettings",
    "MQInstanceConfig",
    "MessageQueueSettings",
    "MigrationSettings",
    "RPCClientSettings",
    "RPCServiceSettings",
    "SchedulerSettings",
    "ServerSettings",
    "ServiceSettings",
    "StorageInstanceConfig",
    "StorageSettings",
    "TaskSettings",
]




"""服务类型常量。

定义服务运行模式的枚举。
"""

from __future__ import annotations

from enum import Enum


class ServiceType(str, Enum):
    """服务类型枚举（应用层配置）。

    用于区分不同的服务运行模式：
    - API: 运行 API 服务（可伴随调度器）
    - WORKER: 运行任务队列 Worker
    - SCHEDULER: 仅运行调度器（独立 pod）
    """

    API = "api"
    WORKER = "worker"
    SCHEDULER = "scheduler"


__all__ = [
    "ServiceType",
]


